# pylint: disable=missing-docstring
# -*- coding: utf-8 -*-

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import argparse
import json
import os
import re
import subprocess
import unittest

import mock
from icetea_lib.IceteaManager import IceteaManager, ExitCodes
from icetea_lib.TestSuite.TestSuite import SuiteException
from icetea_lib.tools.tools import IS_PYTHON3


class MockResults(object):
    def __init__(self, fails=0, inconcs=0):
        self.fails = fails
        self.inconcs = inconcs

    def failure_count(self):
        return self.fails

    def inconclusive_count(self):
        return self.inconcs

    def __len__(self):
        return self.fails + self.inconcs


class IceteaManagerTestcase(unittest.TestCase):

    def setUp(self):
        self.args_noprint = argparse.Namespace(
            available=False, version=False, bin=None,
            binary=False, channel=None,
            clean=True, cloud=False, component=False,
            device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False,
            iface=None, kill_putty=False, list=True,
            listsuites=False, log='./log', my_duts=None,
            nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite=False, tc='all', tc_cfg=None,
            tcdir="test", testtype=False, type=None,
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False,
            repeat=0, platform_name=None, json=False)
        self.args_tc_no_exist = argparse.Namespace(
            available=False, version=False, bin=None,
            binary=False, channel=None,
            clean=True, cloud=False, component=False,
            device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False,
            iface=None, kill_putty=False, list=True,
            listsuites=False, log='./log', my_duts=None,
            nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True,
            skip_case=False, skip_rampdown=False, skip_rampup=False,
            status=False, suite=False, tc='does_not_exist',
            tc_cfg=None, tcdir="test", testtype=False, type=None,
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False,
            repeat=0, platform_name=None, json=False)
        self.args_suite = argparse.Namespace(
            available=False, version=False, bin=None,
            binary=False, channel=None,
            clean=True, cloud=False, component=False,
            device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False,
            iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None,
            nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite="dummy_suite.json", tc=None,
            tc_cfg=None, tcdir="examples", testtype=False, type=None,
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False, repeat=2,
            suitedir="./test/suites", platform_name=None, json=False)
        self.args_tc = argparse.Namespace(
            available=False, version=False, bin=None,
            binary=False, channel=None,
            clean=True, cloud=False, component=False,
            device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False,
            iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None,
            nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite=None, tc="test_cmdline",
            tc_cfg=None, tcdir="examples", testtype=False, type="process",
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False, repeat=2,
            suitedir="./test/suites", forceflash_once=False, forceflash=False,
            ignore_invalid_params=True, failure_return_value=False, stop_on_failure=False,
            branch="", platform_name=None, json=False)

    def tearDown(self):
        if os.path.exists("test_suite.json"):
            os.remove("test_suite.json")

    def test_list_suites(self):
        table = IceteaManager.list_suites(suitedir="./test/suites")
        tab = "+------------------------+\n|    Testcase suites     " \
              "|\n+------------------------+\n" \
              "|    dummy_suite.json    |\n|   failing_suite.json   |\n|  malformed_suite.json  " \
              "|\n| suite_missing_one.json |\n|   working_suite.json   |\n" \
              "+------------------------+"
        self.assertEqual(table.get_string(), tab)

    @mock.patch("icetea_lib.IceteaManager._cleanlogs")
    @mock.patch("icetea_lib.IceteaManager.TestSuite")
    def test_run(self, mock_suite, mock_clean):
        ctm = IceteaManager()

        # Testing different return codes
        with mock.patch.object(ctm, "runtestsuite") as mock_method:
            mock_method.return_value = MockResults()
            retval = ctm.run(args=self.args_tc)
            self.assertEqual(retval, 0)
            self.args_tc.failure_return_value = True
            retval = ctm.run(args=self.args_tc)
            self.assertEqual(retval, 0)
            mock_method.return_value = MockResults(fails=1)
            retval = ctm.run(args=self.args_tc)
            self.assertEqual(retval, 2)
            mock_method.return_value = MockResults(inconcs=1)
            retval = ctm.run(args=self.args_tc)
            self.assertEqual(retval, 3)

        self.args_tc.list = True
        self.args_tc.cloud = False
        mock_suite.list_testcases = mock.MagicMock()
        mock_suite.list_testcases.return_value = "test_list"
        # Test list branch
        retval = ctm.run(args=self.args_tc)
        self.assertEqual(retval, 0)
        self.args_tc.list = False

        self.args_tc.listsuites = True
        ctm.list_suites = mock.MagicMock()
        ctm.list_suites.return_value = "Test-list-item"
        retval = ctm.run(args=self.args_tc)
        self.assertEqual(retval, 0)

    @mock.patch("icetea_lib.IceteaManager.TestSuite")
    def test_run_exceptions(self, mock_suite):
        ctm = IceteaManager()
        mock_suite.side_effect = [SuiteException]
        self.assertEqual(ctm.run(args=self.args_tc), 3)
        with mock.patch.object(ctm, "list_suites") as mock_method:
            mock_method.return_value = None
            self.args_tc.listsuites = True
            self.assertEqual(ctm.run(args=self.args_tc), 2)

    def test_run_returncodes(self):

        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc test_run_retcodes_fail "
                                  "--tcdir test --type process",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)

        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc test_run_retcodes_success "
                                  "--tcdir test --type process",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)

        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc test_run_retcodes_fail "
                                  "--failure_return_value --tcdir test "
                                  "--type process",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_FAIL)

        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--suite working_suite --suitedir test/suites "
                                  "--tcdir examples",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)
        retcode = subprocess.call("python icetea.py --tc test_run_retcodes_notfound -s "
                                  "--tcdir test --type process",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)

        # Run with --clean to clean up
        retcode = subprocess.call(  # pylint: disable=unused-variable
            "python icetea.py --clean ",
            shell=True)

    def test_run_many_cases_one_file(self):
        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc all "
                                  "--tcdir test/tests/multiple_in_one_file",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)

        retcode = subprocess.call(
            "python icetea.py --clean -s "
            "--tc passing_case --failure_return_value "
            "--tcdir test/tests/multiple_in_one_file",
            shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS)

        retcode = subprocess.call(
            "python icetea.py --clean -s "
            "--tc all --failure_return_value "
            "--tcdir test/tests/multiple_in_one_file",
            shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_FAIL)

        retcode = subprocess.call(
            "python icetea.py --clean -s "
            "--tc fail_case --failure_return_value "
            "--tcdir test/tests/multiple_in_one_file",
            shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_FAIL)

        # Run with --clean to clean up
        retcode = subprocess.call(
            "python icetea.py --clean -s",
            shell=True)

    @mock.patch("icetea_lib.IceteaManager.shutil")
    def test_clean(self, mock_shutil):  # pylint: disable=unused-argument
        ctm = IceteaManager()
        self.args_tc.tc = None
        self.args_tc.clean = True
        self.assertEqual(ctm.run(self.args_tc), ExitCodes.EXIT_SUCCESS)

    @mock.patch("icetea_lib.IceteaManager.get_fw_version")
    def test_version_print(self, mock_fw):
        mock_fw.return_value = "1.0.0"
        ctm = IceteaManager()
        self.args_tc.version = True
        self.assertEqual(ctm.run(self.args_tc), ExitCodes.EXIT_SUCCESS)

    def test_platform_name_inconc(self):
        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc test_run_retcodes_success "
                                  "--tcdir test --type process --platform_name TEST_PLAT2 "
                                  "--failure_return_value",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_INCONC, "Non-inconclusive returncode returned. "
                                                          "Allowed_platforms and platform_name "
                                                          "broken.")
        # Run with --clean to clean up
        retcode = subprocess.call(
            "python icetea.py --clean ",
            shell=True)

    def test_platform_name_success(self):
        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc test_run_retcodes_success "
                                  "--tcdir test --type process --platform_name TEST_PLAT "
                                  "--failure_return_value",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS, "Non-success returncode returned. "
                                                           "Allowed_platforms and platform_name "
                                                           "broken.")
        # Run with --clean to clean up
        retcode = subprocess.call(
            "python icetea.py --clean ",
            shell=True)

    def test_reportcmdfail(self):
        retcode = subprocess.call("python icetea.py --clean -s "
                                  "--tc cmdfailtestcase "
                                  "--tcdir test "
                                  "--failure_return_value",
                                  shell=True)
        self.assertEquals(retcode, ExitCodes.EXIT_SUCCESS, "Non-success returncode returned. "
                                                           "ReportCmdFail functionality broken?")
        # Run with --clean to clean up
        retcode = subprocess.call(
            "python icetea.py --clean ",
            shell=True)

    def test_fail_suite_out_regression(self):
        proc = subprocess.Popen(["python", "icetea.py", "--clean", "--suite",
                                 "failing_suite.json", "--suitedir", "test/suites", "--tcdir",
                                 "test/tests", "-s"], stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, _ = proc.communicate()
        self.assertTrue(re.search(b"This is a failing test case", output))

    def test_list_json_output(self):
        self.maxDiff = None
        expected_test_path = os.path.abspath(os.path.join(__file__, "..", "tests",
                                                          "json_output_test",
                                                          "json_output_test_case.py"))
        expected_output = [
            {u"status": u"development",
             u"requirements": {
                 u"duts": {
                     u"*": {
                         u"application": {u"bin": None}}
                 },
                 u"external": {
                     u"apps": []}
             },
             u"filepath": expected_test_path,
             u"name": u"json_output_test",
             u"title": u"Test list output as json",
             u"component": [u"Icetea_ut"],
             u"compatible": {
                 u"framework": {
                     u"version": u">=1.0.0-rc1",
                     u"name": u"Icetea"},
                 u"hw": {u"value": True},
                 u"automation": {u"value": True}
             },
             u"purpose": u"dummy",
             u"type": u"acceptance",
             u"sub_type": None}]

        proc = subprocess.Popen(["python", "icetea.py", "--list", "--tcdir",
                                 "test{}tests{}json_output_test".format(os.path.sep, os.path.sep),
                                 "-s", "--json"],
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)

        output, _ = proc.communicate()
        output = output.rstrip(b"\n")
        if IS_PYTHON3:
            output = output.decode("utf-8")
        self.assertDictEqual(expected_output[0], json.loads(output)[0])

    def test_list_export_to_suite(self):
        expected_call = json.dumps({"default": {}, "testcases": [{"name": "json_output_test"}]})
        proc = subprocess.Popen(["python", "icetea.py", "--list", "--tcdir",
                                 "test{}tests{}json_output_test".format(os.path.sep, os.path.sep),
                                 "-s", "--json", "--export", "test_suite.json"],
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        output, _ = proc.communicate()
        with open("test_suite.json", "r") as file_handle:
            read_data = file_handle.read()
        self.assertDictEqual(json.loads(expected_call), json.loads(read_data))


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re

from icetea_lib.Events.Generics import Observer
from icetea_lib.tools.tools import IS_PYTHON3


class EventMatcher(Observer):
    def __init__(self, event_type, match_data, caller=None,
                 flag=None, callback=None):
        Observer.__init__(self)
        self.caller = caller
        self.event_type = event_type
        self.flag_to_set = flag
        self.callback = callback
        self.match_data = match_data
        self.observe(event_type, self._event_received)

    def _event_received(self, ref, data):
        if self._resolve_match_data(ref, data):
            if self.flag_to_set:
                self.flag_to_set.set()
            if self.callback:
                self.callback(ref, data)
            self.forget()

    def _resolve_match_data(self, ref, event_data):
        """
        If match_data is prefixed with regex: compile it to a regular expression pattern.
        Match event data with match_data as either regex or string.
        :param ref: Reference to object that generated this event.
        :param event_data: Data from event, as string.
        :return: return re.MatchObject if match found, False if ref is not caller
        set for this Matcher or if no match was found.
        """
        if self.caller is None:
            pass
        elif ref is not self.caller:
            return False
        try:
            dat = event_data if IS_PYTHON3 else event_data.decode("utf-8")
            if self.match_data.startswith("regex:"):
                splt = self.match_data.split(":", 1)
                pttrn = re.compile(splt[1])
                match = re.search(pttrn, dat)
                return match if match is not None else False
            return event_data if self.match_data in dat else False
        except UnicodeDecodeError:
            dat = repr(event_data)
            return self._resolve_match_data(ref, dat)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

IceteaManager module. Contains IceteaManager class and some helpers.
IceteaManager is the handler of the entire run. It determines return codes
for the caller and handles processing of arguments as well as triggers the run with the
correct parameters etc.
"""

# Ignore "Use % formatting in logging functions and pass the % parameters as arguments"
# Also ignore "class has no __init__ method" and "Too few public methods" warnings
# Disable Old-style class warning and "Unnecessary parens after 'print' kwd" warning
# pylint: disable=W1202,W0232,R0903,C1001,C0325

import json
import sys
import shutil
import six
import os


import icetea_lib.LogManager as LogManager
from icetea_lib.ResourceProvider.ResourceProvider import ResourceProvider
from icetea_lib.TestSuite.TestSuite import TestSuite, SuiteException
from icetea_lib.TestSuite.TestcaseContainer import TestStatus
from icetea_lib.ResultList import ResultList
from icetea_lib.arguments import get_base_arguments, get_parser, get_tc_arguments
from icetea_lib.tools.tools import Singleton, get_fw_version
from icetea_lib.cloud import Cloud
from icetea_lib.Plugin.PluginManager import PluginManager


def _clean_onerror(func, path, excinfo):
    """
    Error handler for cleaner
    """
    print("%s encountered error when processing %s: %s" % (func, path, excinfo))


def _cleanlogs(silent=False, log_location="log"):
    """
    Cleans up Mbed-test default log directory.

    :param silent: Defaults to False
    :param log_location: Location of log files, defaults to "log"
    :return: Nothing
    """
    try:
        print("cleaning up Icetea log directory.")
        shutil.rmtree(log_location, ignore_errors=silent,
                      onerror=None if silent else _clean_onerror)
    except OSError as error:
        print(error)


class ExitCodes:
    """
    Console exit codes
    """
    EXIT_SUCCESS = 0
    EXIT_ERROR = 1
    EXIT_FAIL = 2
    EXIT_INCONC = 3


@six.add_metaclass(Singleton)
class TCMetaSchema(object):
    """
    Singleton metadata schema object.
    """
    __metaclass__ = Singleton

    def __init__(self, libpath="./icetea_lib"):
        with open(os.path.join(libpath, 'tc_schema.json')) as data_file:
            self._tc_meta_schema = json.load(data_file)

    def get_meta_schema(self):
        """
        Getter for tc meta schema.

        :return: tc_meta_schema
        """
        return self._tc_meta_schema


class IceteaManager(object):
    """
    IceteaManager class. This is the master of the entire run. The primary entry point into
    execution is the run method.
    """
    def __init__(self):
        """
        Constructor for IceteaManager. Appends libraries to sys.path, loads the test case
        metadata schema, parses arguments and initializes logging.
        """
        self.libpath = os.sep.join(os.path.abspath(__file__).split(os.sep)[:-1])
        sys.path.append(self.libpath)
        libpath2 = os.sep.join(self.libpath.split(os.sep)[:-1])
        sys.path.append(libpath2)
        # Initialize TCMetaSchema with correct libpath
        TCMetaSchema(self.libpath)
        self.args, self.unknown = IceteaManager._parse_arguments()
        # If called with --clean, clean up logs.
        if self.args.clean:
            _cleanlogs(silent=self.args.silent, log_location=self.args.log)

        LogManager.init_base_logging(self.args.log, verbose=self.args.verbose,
                                     silent=self.args.silent, color=self.args.color,
                                     no_file=(self.args.list or self.args.listsuites),
                                     truncate=not self.args.disable_log_truncate)

        self.logger = LogManager.get_logger("icetea")
        self.pluginmanager = None
        self.resourceprovider = ResourceProvider(self.args)
        self._init_pluginmanager()
        self.resourceprovider.set_pluginmanager(self.pluginmanager)

    @staticmethod
    def list_suites(suitedir="./testcases/suites", cloud=False):
        """
        Static method for listing suites from both local source and cloud.
        Uses PrettyTable to generate the table.

        :param suitedir: Local directory for suites.
        :param cloud: cloud module
        :return: PrettyTable object or None if no test cases were found
        """
        suites = []
        suites.extend(TestSuite.get_suite_files(suitedir))

        # no suitedir, or no suites -> append cloud.get_campaigns()

        if cloud:
            names = cloud.get_campaign_names()
            if names:
                suites.append("------------------------------------")
                suites.append("FROM CLOUD:")
                suites.extend(names)
        if not suites:
            return None

        from prettytable import PrettyTable
        table = PrettyTable(["Testcase suites"])
        for suite in suites:
            table.add_row([suite])
        return table

    @staticmethod
    def _parse_arguments():
        """
        Static method for paring arguments
        """
        parser = get_base_arguments(get_parser())
        parser = get_tc_arguments(parser)
        args, unknown = parser.parse_known_args()
        return args, unknown

    def check_args(self):
        """
        Validates that a valid number of arguments were received and that all arguments were
        recognised.

        :return: True or False.
        """
        parser = get_base_arguments(get_parser())
        parser = get_tc_arguments(parser)
        # Disable "Do not use len(SEQ) as condition value"
        # pylint: disable=C1801
        if len(sys.argv) < 2:
            self.logger.error("Icetea called with no arguments! ")
            parser.print_help()
            return False
        elif not self.args.ignore_invalid_params and self.unknown:
            self.logger.error("Unknown parameters received, exiting. "
                              "To ignore this add --ignore_invalid_params flag.")
            self.logger.error("Following parameters were unknown: {}".format(self.unknown))
            parser.print_help()
            return False
        return True

    def _init_pluginmanager(self):
        """
        Initialize PluginManager and load run wide plugins.
        """
        self.pluginmanager = PluginManager(logger=self.logger)
        self.logger.debug("Registering execution wide plugins:")
        self.pluginmanager.load_default_run_plugins()
        self.pluginmanager.load_custom_run_plugins(self.args.plugin_path)
        self.logger.debug("Execution wide plugins loaded and registered.")

    def run(self, args=None):
        """
        Runs the set of tests within the given path.
        """
        # Disable "Too many branches" and "Too many return statmenets" warnings
        # pylint: disable=R0912,R0911
        retcodesummary = ExitCodes.EXIT_SUCCESS
        self.args = args if args else self.args

        if not self.check_args():
            return retcodesummary

        if self.args.clean:
            if not self.args.tc and not self.args.suite:
                return retcodesummary

        # If called with --version print version and exit
        version = get_fw_version()
        if self.args.version and version:
            print(version)
            return retcodesummary
        elif self.args.version and not version:
            print("Unable to get version. Have you installed Icetea correctly?")
            return retcodesummary

        self.logger.info("Using Icetea version {}".format(version) if version
                         else "Unable to get Icetea version. Is Icetea installed?")

        # If cloud set, import cloud, get parameters from environment, initialize cloud
        cloud = self._init_cloud(self.args.cloud)

        # Check if called with listsuites. If so, print out suites either from cloud or from local
        if self.args.listsuites:
            table = self.list_suites(self.args.suitedir, cloud)
            if table is None:
                self.logger.error("No suites found!")
                retcodesummary = ExitCodes.EXIT_FAIL
            else:
                print(table)
            return retcodesummary

        try:
            testsuite = TestSuite(logger=self.logger, cloud_module=cloud, args=self.args)
        except SuiteException as error:
            self.logger.error("Something went wrong in suite creation! {}".format(error))
            retcodesummary = ExitCodes.EXIT_INCONC
            return retcodesummary

        if self.args.list:
            if self.args.cloud:
                testsuite.update_testcases()
            testcases = testsuite.list_testcases()
            print(testcases)
            return retcodesummary

        results = self.runtestsuite(testsuite=testsuite)

        if not results:
            retcodesummary = ExitCodes.EXIT_SUCCESS
        elif results.failure_count() and self.args.failure_return_value is True:
            retcodesummary = ExitCodes.EXIT_FAIL
        elif results.inconclusive_count() and self.args.failure_return_value is True:
            retcodesummary = ExitCodes.EXIT_INCONC

        return retcodesummary

    def runtestsuite(self, testsuite):
        """
        Runs a single test suite

        :param testsuite: TestSuite
        :return: ResultList
        """
        if testsuite.status == TestStatus.READY:
            results = testsuite.run()
        else:
            results = ResultList()
        #Disable "Expression is assigned to nothing" warning
        #pylint: disable=W0106
        [handler.flush() for handler in self.logger.handlers]
        results.save(heads={'Build': '', 'Branch': self.args.branch})
        sys.stdout.flush()
        self._cleanup_resourceprovider()
        return results

    #Disable "String statement has no effect" warning
    #pylint: disable=W0105
    """
        PRIVATE FUNCTIONS HERE
    """

    def _cleanup_resourceprovider(self):
        """
        Calls cleanup for ResourceProvider of this run.

        :return: Nothing
        """
        #Disable too broad exception warning
        #pylint: disable=W0703
        self.resourceprovider = ResourceProvider(self.args)
        try:
            self.resourceprovider.cleanup()
            self.logger.info("Cleanup done.")
        except Exception as error:
            self.logger.error("Cleanup failed! %s", error)

    def _init_cloud(self, cloud_arg):
        """
        Initializes Cloud module if cloud_arg is set.

        :param cloud_arg: taken from args.cloud
        :return: cloud module object instance
        """
        # Disable too broad exception warning
        # pylint: disable=W0703
        cloud = None
        if cloud_arg:
            try:
                if hasattr(self.args, "cm"):
                    cloud_module = self.args.cm if self.args.cm else None
                    self.logger.info("Creating cloud module {}.".format(cloud_module))
                else:
                    cloud_module = None

                cloud = Cloud(host=None, module=cloud_module, logger=self.logger, args=self.args)
            except Exception as error:
                self.logger.warning("Cloud module could not be initialized: {}".format(error))
                cloud = None
        return cloud



# pylint: disable=missing-docstring,protected-access

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from icetea_lib.Result import Result
from icetea_lib.cloud import Cloud, create_result_object


class CloudTestcase(unittest.TestCase):

    def setUp(self):
        with mock.patch("icetea_lib.cloud.get_pkg_version") as mocked_reader:
            mocked_reader.return_value = [mock.MagicMock()]
            self.cloudclient = Cloud(module="moduletest")

    @mock.patch("icetea_lib.cloud.get_pkg_version")
    def test_module_import(self, mocked_version_reader):
        mocked_version_reader.return_value = "1.0.0"
        # Test ImportError with non-existent module
        with self.assertRaises(ImportError):
            cloud_module = Cloud(module="Nonexistentmodule")

        # Verify that Cloud interface init function can import cloud module properly
        cloud_module = Cloud(module="test.moduletest")
        self.assertIsNotNone(cloud_module._client, "Cloud client was None!")

        # Verify correct behaviour with incorrect module
        with self.assertRaises(ImportError):
            cloud_module = Cloud(module="test.moduletest.wrongmodule")

    def test_get_suite(self):
        self.cloudclient.get_suite("test_suite", ["foo", "bar"])
        self.cloudclient._client.get_suite.assert_called_once_with("test_suite", ["foo", "bar"])

    def test_get_campaigns(self):
        self.cloudclient.get_campaigns()
        self.assertTrue(self.cloudclient._client.get_campaigns.called)

    def test_get_campaign_id(self):
        self.cloudclient.get_campaign_id("test_campaign")
        self.cloudclient._client.get_campaign_id.assert_called_once_with("test_campaign")
        with self.assertRaises(KeyError):
            self.cloudclient.get_campaign_id("test_campaign")

    def test_get_campaign_names(self):
        self.cloudclient.get_campaign_names()
        self.assertTrue(self.cloudclient._client.get_campaign_names.called)

    def test_update_testcase(self):
        self.cloudclient.update_testcase({"test": "meta", "meta": "data"})
        self.cloudclient._client.update_testcase.assert_called_once_with({"test": "meta",
                                                                          "meta": "data"})

    def test_send_result(self):
        self.cloudclient.send_result({"verdict": "PASS"})
        self.cloudclient._client.upload_results.assert_called_once_with({"verdict": "PASS"})

        self.assertIsNone(self.cloudclient.send_result("data"))

    def test_converter(self):
        metadata = {"status": "ready",
                    "name": "tc_name",
                    "title": "this_is_title",
                    "requirements": {
                    },
                    "feature": "unit test",
                    "component": "Icetea"
                   }

        self.assertEquals(self.cloudclient._convert_to_db_tc_metadata(metadata),
                          {'status': {'value': 'ready'}, 'requirements': {'node': {'count': 1}},
                           'other_info': {'features': 'unit test', 'components': 'Icetea',
                                          'title': 'this_is_title'},
                           'tcid': 'tc_name'})

    def test_cloudresult_simple(self):
        test_res = {
            "testcase": "tc_name",
            "verdict": "pass",
            "retcode": 0,
            "fw_name": "Icetea",
            "fw_version": "0.10.2"
        }
        result = Result(test_res)
        res = create_result_object(result)
        compare = {
            'exec': {
                'verdict': 'pass',
                'env': {
                    'framework': {
                        'ver': '0.10.2',
                        'name': 'Icetea'
                    }
                },
                'dut': {
                    'sn': 'unknown'
                }
            },
            'tcid': 'tc_name'
        }

        self.assertDictContainsSubset(compare, res)

    def test_cloudresult_full(self):
        test_res = {
            "testcase": "tc_name",
            "verdict": "fail",
            "reason": "ohnou",
            "retcode": 0,
            "duration": 1,
            "fw_name": "Icetea",
            "fw_version": "0.10.2"
        }
        result = Result(test_res)

        class DummyBuild(object):
            @property
            def branch(self):
                return 'master'

            @property
            def commit_id(self):
                return '1234'

            @property
            def build_url(self):
                return 'url'

            @property
            def giturl(self):
                return 'url'

            @property
            def date(self):
                return "22.22.2222"

            @property
            def sha1(self):
                return "asdv"

        class DummyDut(object):
            @property
            def resource_id(self):
                return '123'

            @property
            def platform(self):
                return 'K64F'

            @property
            def build(self):
                return DummyBuild()

        result.set_dutinformation([DummyDut()])
        res = create_result_object(result)
        compare = {
            'exec': {
                'verdict': 'pass',
                'note': 'ohnou',
                'duration': 1,
                'env': {
                    'framework': {
                        'ver': '0.10.2',
                        'name': 'Icetea'
                    }
                },
                'sut': {
                    'commitId': '1234',
                    'buildUrl': 'url',
                    'gitUrl': 'url',
                    'branch': 'master',
                    "buildDate": "22.22.2222",
                    "buildSha1": "asdv"
                },
                'dut': {
                    'sn': '123',
                    'model': 'K64F'
                }
            },
            'tcid': 'tc_name'
        }
        self.assertDictContainsSubset(compare, res)



#!/usr/bin/env python
"""Enhanced Serial Port class
part of pyserial (http://pyserial.sf.net)  (C)2002 cliechti@gmx.net

another implementation of the readline and readlines method.
this one should be more efficient because a bunch of characters are read
on each access, but the drawback is that a timeout must be specified to
make it work (enforced by the class __init__).

this class could be enhanced with a read_until() method and more
like found in the telnetlib.
"""
import re
import pkg_resources
from threading import Lock
from serial import Serial, SerialException, SerialTimeoutException

class EnhancedSerial(Serial):
    def __init__(self, *args, **kwargs):
        #ensure that a reasonable timeout is set
        timeout = kwargs.get('timeout',0.1)
        if timeout < 0.01: timeout = 0.1
        kwargs['timeout'] = timeout
        Serial.__init__(self, *args, **kwargs)
        self.buffer_lock = Lock()
        self.buf = ''
        self.pyserial_version = self.get_pyserial_version()
        self.is_pyserial_v3 = self.pyserial_version >= 3.0

    def get_pyserial_version(self):
        """! Retrieve pyserial module version
        @return Returns float with pyserial module number
        """
        self.re_float = re.compile("^\d+\.\d+")
        pyserial_version = pkg_resources.require("pyserial")[0].version
        version = 3.0
        m = self.re_float.search(pyserial_version)
        if m:
            try:
                version = float(m.group(0))
            except ValueError:
                version = 3.0   # We will assume you've got latest (3.0+)
        return version

    def safe_sendBreak(self):
        """! Closure for pyserial version dependant API calls
        """
        if self.is_pyserial_v3:
            return self._safe_sendBreak_v3_0()
        return self._safe_sendBreak_v2_7()

    def _safe_sendBreak_v2_7(self):
        """! pyserial 2.7 API implementation of sendBreak/setBreak
        @details
        Below API is deprecated for pyserial 3.x versions!
        http://pyserial.readthedocs.org/en/latest/pyserial_api.html#serial.Serial.sendBreak
        http://pyserial.readthedocs.org/en/latest/pyserial_api.html#serial.Serial.setBreak
        """
        result = True
        try:
            self.sendBreak()
        except:
            # In Linux a termios.error is raised in sendBreak and in setBreak.
            # The following setBreak() is needed to release the reset signal on the target mcu.
            try:
                self.setBreak(False)
            except:
                result = False
        return result

    def _safe_sendBreak_v3_0(self):
        """! pyserial 3.x API implementation of send_brea / break_condition
        @details
        http://pyserial.readthedocs.org/en/latest/pyserial_api.html#serial.Serial.send_break
        http://pyserial.readthedocs.org/en/latest/pyserial_api.html#serial.Serial.break_condition
        """
        result = True
        try:
            self.send_break()
        except:
            # In Linux a termios.error is raised in sendBreak and in setBreak.
            # The following break_condition = False is needed to release the reset signal on the target mcu.
            self.break_condition = False
        return result
        
    def readline(self, maxsize=None, timeout=1):
        """maxsize is ignored, timeout in seconds is the max time that is way for a complete line"""
        tries = 0
        while 1:
            try:
                block = self.read(512)
                if isinstance(block, bytes):
                    block = block.decode()
                elif isinstance(block, str):
                    block = block.decode()
                else:
                    raise ValueError("Unknown data")
            except SerialTimeoutException:
                # Exception that is raised on write timeouts.
                block = ''
            except SerialException:
                # In case the device can not be found or can not be configured.
                block = ''
            except ValueError:
                # Will be raised when parameter are out of range, e.g. baud rate, data bits.
                # UnicodeError-Raised when a Unicode-related encoding or decoding error occurs. It is a subclass of ValueError.
                block = ''
            with self.buffer_lock:
                # Let's lock, just in case
                self.buf += block
                pos = self.buf.find('\n')
                if pos >= 0:
                    line, self.buf = self.buf[:pos+1], self.buf[pos+1:]
                    return line
            tries += 1
            if tries * self.timeout > timeout:
                break
        return None

    def peek(self):
        """
        Peek into the buffer to see if there are unfinished lines of data available.

        :return: str
        """
        with self.buffer_lock:
            # Let's lock, just in case.
            return self.buf

    def readlines(self, sizehint=None, timeout=1):
        """read all lines that are available. abort after timout
        when no more data arrives."""
        lines = []
        while 1:
            line = self.readline(timeout=timeout)
            if line:
                lines.append(line)
            if not line or line[-1:] != '\n':
                break
        return lines



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and


Testcase container class for TestSuite.
"""

from jsonmerge import merge
from jsonschema import validate, ValidationError, SchemaError
import semver
import datetime
import traceback
import gc
import time
from inspect import isclass
from six import iteritems

import icetea_lib.LogManager as LogManager
from icetea_lib.bench import ReturnCodes
from icetea_lib.tools.tools import get_fw_version, import_module, load_class
from icetea_lib.arguments import get_parser, get_tc_arguments, get_base_arguments
from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList

# pylint: disable=too-many-arguments,useless-super-delegation,too-many-branches,too-many-statements


class TestStatus:
    """
    Enumeration for test statuses.
    """
    PENDING = 4
    PREPARED = 3
    READY = 2
    RUNNING = 1
    FINISHED = 0


class TestcaseContainer(object):
    def __init__(self, logger=None):
        self.logger = logger
        self.status = None
        self._instance = None
        self._modulename = None
        self._moduleroot = None
        self._final_configuration = {}
        self.tcname = None
        self._meta_schema = None
        self._result = None
        self._filepath = None
        self._suiteconfig = {}
        self._infodict = {}
        if not logger:
            import logging
            self.logger = logging.getLogger("TCContainer")
            if len(self.logger.handlers) == 0:
                self.logger.addHandler(logging.StreamHandler())
                self.logger.setLevel(logging.INFO)

    @staticmethod
    def find_testcases(modulename, moduleroot, tc_meta_schema, path=None, suiteconfig=None,
                       logger=None):
        """
        Static method for generating a list of TestcaseContainer objects from a module.

        :param modulename: Name of module to parse
        :param moduleroot: module root
        :param tc_meta_schema: Schema to use in validation
        :param path: Path to module file
        :param suiteconfig: Optional configuration dictionary from suite
        :param logger: Logger
        :return: list of TestcaseContainer instances
        """
        if not isinstance(modulename, str):
            raise TypeError("modulename should be a string")

        if len(modulename) == 0:
            raise ValueError("modulename shouldn't be empty")

        #Try to import the module.
        try:
            module = import_module(modulename)
        except Exception as e:
            if logger:
                logger.debug("Error while importing module {}: {}".format(modulename, e))
            raise ImportError("Error importing module {}: {}".format(modulename, e))

        tclist = []

        for test_class_name, test_class in iteritems(module.__dict__):
            if not isclass(test_class):
                continue
            # if a class as the constant flag IS_TEST set to true or is named Testcase,
            # fulfill test description and add it to the list
            if getattr(test_class, "IS_TEST", False) or test_class_name == "Testcase":
                testcase = TestcaseContainer(logger=logger)
                test_case = test_class()
                testcase.generate_members(modulename, test_case, moduleroot, path,
                                          tc_meta_schema, test_class_name, suiteconfig)
                tclist.append(testcase)
        return tclist

    def __copy__(self):
        cont = TestcaseContainer.find_testcases(self._modulename, self._moduleroot, self._meta_schema, self._filepath,
                                                self._suiteconfig, self.logger)
        for tc in cont:
            if self.tcname == tc.tcname:
                return tc
        return None

    def generate_members(self, modulename, tc_instance, moduleroot, path, meta_schema,
                         test_class_name, suiteconfig=None):
        """
        Setter and generator for internal variables.

        :param modulename: Name of the module
        :param tc_instance: Bench instance
        :param moduleroot: Root folder of the module
        :param path:  Path to module file
        :param meta_schema: Schema used for Validation
        :param test_class_name: Name of the class
        :param suiteconfig: Optional configuration dictionary from suite
        :return: Nothing, modifies objects content in place
        """
        self._modulename = modulename
        self.tcname = tc_instance.get_test_name()
        self.status = TestStatus.PENDING
        self._instance = tc_instance
        self._final_configuration = {}
        self._moduleroot = moduleroot
        self._meta_schema = meta_schema
        self._result = None
        self._filepath = path
        self._suiteconfig = suiteconfig if suiteconfig else {}

        if tc_instance.get_test_component():
            comp = tc_instance.get_test_component()
        else:
            comp = ["None"]
        if tc_instance.get_features_under_test():
            feat = tc_instance.get_features_under_test()
        else:
            feat = ''
        if tc_instance.get_allowed_platforms():
            platforms = tc_instance.get_allowed_platforms()
        else:
            platforms = ''
        self._infodict = {
            'name': self.tcname,
            'path': modulename + "." + test_class_name,
            'status': tc_instance.status() if tc_instance.status() else '',
            'type': tc_instance.type() if tc_instance.type() else '',
            'subtype': tc_instance.subtype() if tc_instance.subtype() else '',
            'group': moduleroot,
            'file': self._filepath,  # path to the file which hold this test
            'comp': comp,
            'feature': feat,
            'allowed_platforms': platforms,
            'fail': ''
        }

    def get_infodict(self):
        return self._infodict

    def get(self, field):
        """
        Gets value of property/configuration field.

        :param field: Name of configuration property to get
        :return: Value of configuration property field. None if not found.
        """
        return self.get_instance().config.get(field)

    def get_instance_config(self):
        """
        Get configuration currently set into the test instance.
        :return: dict
        """
        config = self.get_instance().config
        config["filepath"] = self._filepath
        return config

    def get_final_config(self):
        return self._final_configuration

    def get_suiteconfig(self):
        return self._suiteconfig

    def get_instance(self):
        """
        Getter for testcase Bench instance. If instance does not exist, it will be created.

        :return: Bench instance of this testcase.
        """
        return self._instance

    def get_result(self):
        return self._result

    def get_name(self):
        return self.tcname

    def merge_tc_config(self, conf_to_merge):
        """
        Merges testcase configuration with dictionary conf_to_merge.

        :param conf_to_merge: Dictionary of configuration to
        merge with testcase default configuration
        :return: Nothing
        """
        self._final_configuration = merge(self._final_configuration, conf_to_merge)

    def set_suiteconfig(self, config):
        self._suiteconfig = config

    def set_result(self, result):
        self._result = result

    def set_final_config(self):
        """
        Sets configuration for testcase instance from self._final_configuration field.
        """
        if self._instance:
            self._instance.set_config(self._final_configuration)

    def validate_tc_instance(self):
        """
        Validates this testcase instance metadata and fetches the tc configuration.

        :return Nothing
        :raises SyntaxError
        """
        if not self.validate_testcase_metadata(self.get_instance()):
            raise SyntaxError("Invalid TC metadata")
        self._final_configuration = self.get_instance().get_config()

    def validate_testcase_metadata(self, tc):
        try:
            validate(tc.config, self._meta_schema)
        except ValidationError  as err:
            self.logger.error("Metadata validation failed! Please fix your TC Metadata!")
            self.logger.debug(tc.config)
            self.logger.error(err)
            return False
        except SchemaError as err:
            self.logger.error("Schema error")
            self.logger.error(err)
            return False
        return True

    def run(self, forceflash=False):
        """
        Runs the testcase associated with this container.

        :param forceflash: boolean, True if forceflash should be used
        :return: Result
        """
        if self.status == TestStatus.FINISHED:
            self.logger.debug("Creating new bench instance for repeat.")
            self._instance = self._create_new_bench_instance(self._modulename)
            self.set_final_config()
        self.status = TestStatus.RUNNING
        self.logger.debug("Starting test case {}".format(self.tcname))
        tc_instance = self.get_instance()

        result = self._check_skip(tc_instance)
        if result:
            self.logger.debug("Skipping test case {}".format(self.tcname))
            self._result = result
            self.status = TestStatus.FINISHED
            return result

        # Check if version checking is enabled in cli
        # and if the bench has the compatible key in it's config.
        result = self._check_version(tc_instance)
        if result is not None:
            self.logger.debug("Version check triggered, skipping test case {}.".format(self.tcname))
            self._result = result
            self.status = TestStatus.FINISHED
            return result

        parser = get_tc_arguments(get_base_arguments(get_parser()))
        args, unknown = parser.parse_known_args()
        if len(unknown) > 0:
            for para in unknown:
                self.logger.warning("Icetea received unknown parameter {}".format(para))
            if not args.ignore_invalid_params:
                self.logger.error(
                    "Unknown parameters received, exiting. To ignore this add --ignore_invalid_params flag.")
                parser.print_help()
                result = tc_instance.get_result()
                result.set_verdict(verdict="inconclusive", retcode=-1, duration=0)
                self.status = TestStatus.FINISHED
                return result

        args.forceflash = forceflash
        self.status = TestStatus.RUNNING
        tc_instance.set_args(args)
        self.logger.info("")
        self.logger.info("START TEST CASE EXECUTION: '%s'" % tc_instance.get_test_name())
        self.logger.info("")

        a = datetime.datetime.now()
        try:
            retcode = tc_instance.run()
            self.logger.debug("Test bench returned return code {}".format(retcode))
        except:
            traceback.print_exc()
            retcode = -9999

        b = datetime.datetime.now()

        result = tc_instance.get_result(tc_file=self._filepath)
        # Force garbage collection
        gc.collect()
        # cleanup Testcase
        tc_instance = None
        LogManager.finish_testcase_logging()
        self.status = TestStatus.FINISHED

        if isinstance(result, ResultList):
            self.logger.debug("Received a list of results from test bench.")
            return result

        if result.retcode == ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER:
            print("Press CTRL + C again if you want to abort test run")
            try:
                time.sleep(5)
            except KeyboardInterrupt:
                self.status = TestStatus.FINISHED
                raise

        c = b - a
        duration = c.total_seconds()
        self.logger.debug("Duration: {} seconds".format(duration))

        verdict = None
        if retcode == 0:
            verdict = "pass"
        elif retcode in ReturnCodes.INCONCLUSIVE_RETCODES:
            verdict = "inconclusive"
        elif retcode == ReturnCodes.RETCODE_SKIP:
            verdict = "skip"
        else:
            verdict = "fail"
        result.set_verdict(
            verdict=verdict,
            retcode=retcode,
            duration=duration)
        self._result = result

        return result

    def _create_new_bench_instance(self, modulename):
        module = import_module(modulename)

        for test_class_name, test_class in iteritems(module.__dict__):
            if not isclass(test_class):
                continue
            if getattr(test_class, "IS_TEST", False) is True or test_class_name == "Testcase":
                inst = test_class()
                if inst.get_test_name() == self.tcname:
                    return inst
                else:
                    continue

    def _load_testcase(self, modulename, verbose=False):
        """
        :param modulename: testcase to be loaded
        :param verbose: print exceptions when loading class
        :return: testcase instance
        :raises TypeError exception when modulename is not string
        :raises ImportError exception when cannot load testcase
        """
        if not isinstance(modulename, str):
            raise TypeError("Error, runTest: modulename not a string.")
        try:
            module = load_class(modulename, verbose)
        except ValueError as e:
            raise ImportError("Error, load_testcase: loadClass raised ValueError: {}".format(e))

        if module is None:
            raise ImportError("Error, runTest: "
                              "loadClass returned NoneType for modulename: %s" % modulename)

        return module()

    def _check_skip(self, tc_instance):
        # Skip the TC IF NOT defined on the command line
        if tc_instance.skip():
            info = tc_instance.skip_info()
            if info.get('only_type') or info.get('platforms'):
                # only_type cannot be properly checked here, se we proceed
                # and check the final configuration in Bench.
                return False
            else:
                self.logger.info("TC '%s' will be skipped because of '%s'" % (
                    tc_instance.get_test_name(), (tc_instance.skip_reason())))
                result = tc_instance.get_result()
                result.set_verdict(verdict='skip', retcode=-1, duration=0)
                del tc_instance
                self._result = result
                return result
        else:
            return False

    def _check_major_version(self, fw_version, version_string):
        if int(fw_version[0]) > 0 and version_string[0] == '0':
            return False
        elif int(fw_version[0]) > 0 and version_string[1] == '0':
            return False
        elif int(fw_version[0]) > 0 and version_string[2] == '0':
            return False
        else:
            return True

    def _check_version(self, tc_instance):
        if tc_instance.config.get(
                "compatible") and tc_instance.config['compatible']['framework']['name']:
            framework = tc_instance.config['compatible']['framework']
            # Check if version requirement is available
            # and that the testcase is meant for this framework
            if framework['version'] and framework['name'] == "Icetea":
                ver_str = framework['version']
                fw_version = get_fw_version()
                try:
                    if not self._check_major_version(fw_version, ver_str):
                        result = self._wrong_version(tc_instance, ver_str,
                                                     "Testcase not suitable for version >1.0.0. "
                                                     "Please install Icetea {}".format(ver_str))
                        return result
                except ValueError:
                    # Unable to convert fw_version to integer, let's just proceed.
                    return None
                if ver_str[0].isdigit():
                    return self._wrong_version(
                        tc_instance, ver_str) if fw_version != ver_str else None

                # Handle case where the version is a version number without comparison operators
                if not semver.match(fw_version, ver_str):
                    result = self._wrong_version(tc_instance, ver_str)
                    return result
            return None
        else:
            return None

    def _wrong_version(self, tc_instance, ver_str, msg=None):
        msg = msg if msg else "Version {} of Icetea required".format(ver_str)
        self.logger.info("TC '%s' will be skipped because of '%s'" % (
            tc_instance.get_test_name(), msg))
        result = tc_instance.get_result()
        result.skip_reason = msg
        result.set_verdict(verdict='skip', retcode=-1, duration=0)
        del tc_instance
        return result


class DummyContainer(TestcaseContainer):
    """
    Class DummyContainer

    subclasses TestcaseContainer, acts as a dummy object for listing test cases
    that were not found when importing test cases.

    """

    def __init__(self, logger=None):
        """
        Just initialize the super class.

        :param logger: logger to use.
        """
        super(DummyContainer, self).__init__(logger)

    @staticmethod
    def find_testcases(modulename, moduleroot, tc_meta_schema, path=None, suiteconfig=None,
                       logger=None):
        """
        Static method find_testcases. Returns a DummyContainer with attributes collected from
        function params.
        """
        dc = DummyContainer(logger)
        dc.tcname = modulename
        dc._modulename = modulename
        dc.status = TestStatus.PENDING
        dc._instance = None
        dc._final_configuration = {}
        dc._moduleroot = moduleroot
        dc._meta_schema = tc_meta_schema
        dc._result = None
        dc._filepath = path
        dc._suiteconfig = suiteconfig if suiteconfig else {}
        return dc

    def run(self, forceflash=False):
        """
        Just returns the Result object for this Dummy.
        """
        return self._result

    def get(self, field):
        return None

    def set_final_config(self):
        pass

    def set_result_verdict(self, reason):
        """
        Sets the inconclusive verdict for this DummyContainer with reason reason.

        :param reason: String reason for why this dummy exists.
        :return: Nothing
        """
        if not self._result:
            self._result = Result()

        self._result.set_verdict(verdict="inconclusive",
                                 retcode=ReturnCodes.RETCODE_FAIL_INCONCLUSIVE,
                                 duration=0)
        self._result.tc_metadata["name"] = self.tcname
        self._result.fail_reason = reason

    def validate_tc_instance(self):
        return True



# pylint: disable=missing-docstring,pointless-string-statement

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import unittest
import sys

import xml.etree.ElementTree as ET

from lxml import html

try:
    # python2
    from StringIO import StringIO
except ImportError:
    # python3
    from io import StringIO

from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList
from icetea_lib.Reports import ReportConsole, ReportHtml, ReportJunit
from icetea_lib.tools.tools import hex_escape_str


class ReportCase(unittest.TestCase):

    """
        One unittest TestCase for all report types.
    """

    """
        ReportConsole unit tests
    """
    def test_reportconsole_no_results(self):
        saved_stdout = sys.stdout
        results = ResultList()
        try:
            out = StringIO()
            sys.stdout = out
            report = ReportConsole(results)
            report.generate()
            output = out.getvalue().strip()
            lines = output.split("\n")
            self.assertEqual(len(lines), 12)
        finally:
            sys.stdout = saved_stdout

    def test_reportconsole_one_results(self):
        saved_stdout = sys.stdout
        results = ResultList()
        results.append(Result())
        try:
            out = StringIO()
            sys.stdout = out
            report = ReportConsole(results)
            report.generate()
            output = out.getvalue().strip()
            lines = output.split("\n")
            self.assertEqual(len(lines), 14)
            self.assertRegexpMatches(lines[8], r"Final Verdict.*INCONCLUSIVE", lines[8])
            self.assertRegexpMatches(lines[9], r"count.*1", lines[9])
            self.assertRegexpMatches(lines[10], r"passrate.*0.00 \%", lines[10])
        finally:
            sys.stdout = saved_stdout

    def test_reportconsole_multiple_results(self):  # pylint: disable=invalid-name
        saved_stdout = sys.stdout
        results = ResultList()
        results.append(Result())
        results.append(Result())
        results.append(Result())
        try:
            out = StringIO()
            sys.stdout = out
            report = ReportConsole(results)
            report.generate()
            output = out.getvalue().strip()
            lines = output.split("\n")
            self.assertEqual(len(lines), 16)
            self.assertRegexpMatches(lines[10], r"Final Verdict.*INCONCLUSIVE", lines[10])
            self.assertRegexpMatches(lines[11], r"count.*3", lines[11])
            self.assertRegexpMatches(lines[12], r"passrate.*0.00 \%", lines[12])

        finally:
            sys.stdout = saved_stdout

    def test_reportconsole_skip(self):
        saved_stdout = sys.stdout
        results = ResultList()
        res = Result()
        res.skip_reason = "Skip_reason"
        res.set_verdict("skip", -1, -1)
        results.append(res)
        try:
            out = StringIO()
            sys.stdout = out
            report = ReportConsole(results)
            report.generate()
            output = out.getvalue().strip()
            lines = output.split("\n")
            self.assertEqual(len(lines), 14)
            self.assertRegexpMatches(lines[3], r"skip.*Skip_reason")
            self.assertRegexpMatches(lines[8], r"Final Verdict.*PASS", lines[8])
            self.assertRegexpMatches(lines[9], r"count.*1", lines[9])
            self.assertRegexpMatches(lines[10], r"passrate.*0.00 \%", lines[10])
            self.assertRegexpMatches(lines[11], r"skip.*1", lines[10])
        finally:
            sys.stdout = saved_stdout

    def test_reportconsole_decodefail(self):
        saved_stdout = sys.stdout
        failing_message = "\x00\x00\x00\x00\x00\x00\x01\xc8"
        results = ResultList()
        res = Result()
        res.set_verdict("fail", 1001, 0)
        res.fail_reason = failing_message
        results.append(res)
        try:
            out = StringIO()
            sys.stdout = out
            report = ReportConsole(results)
            report.generate()
            output = out.getvalue().strip()
            lines = output.split("\n")

            self.assertEqual(len(lines), 14)
            self.assertRegexpMatches(lines[8], r"Final Verdict.*FAIL", lines[8])
            self.assertRegexpMatches(lines[9], r"count.*1", lines[9])
            self.assertRegexpMatches(lines[10], r"passrate.*0.00 \%", lines[10])
        finally:
            sys.stdout = saved_stdout

    """
        ReportJunit tests
    """

    def test_junit_default(self):
        str_should_be = '<testsuite failures="0" tests="1" errors="0" skipped="0">\n\
            <testcase classname="test-case-A1" name="unknown" time="20"></testcase>\n\
        </testsuite>'
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "PASS", "duration": 20}))
        junit = ReportJunit(results)
        str_report = junit.to_string()
        xml_shouldbe = ET.fromstring(str_should_be)
        report_xml = ET.fromstring(str_report)
        self.assertDictEqual(xml_shouldbe.attrib, report_xml.attrib)
        self.assertDictEqual(xml_shouldbe.find("testcase").attrib, report_xml.find(
            "testcase").attrib)
        self.assertEqual(len(xml_shouldbe.findall("testcase")), len(report_xml.findall(
            "testcase")))

    def test_junit_multiple(self):
        str_should_be = '<testsuite failures="2" tests="7" errors="1" skipped="1">\n\
            <testcase classname="test-case-A1" name="unknown" time="20"></testcase>\n\
            <testcase classname="test-case-A2" name="unknown" time="50"></testcase>\n\
            <testcase classname="test-case-A3" name="unknown" time="120"></testcase>\n\
            <testcase classname="test-case-A4" name="unknown" time="120">\n\
                <failure message="unknown"></failure>\n\
            </testcase>\n\
            <testcase classname="test-case-A5" name="unknown" time="1">\n\
                <skipped></skipped>\n\
            </testcase>\n\
            <testcase classname="test-case-A6" name="unknown" time="2">\n\
                <error message="unknown"></error>\n\
            </testcase>\n\
            <testcase classname="test-case-A4" name="unknown" time="1220">\n\
                <failure message="WIN blue screen"></failure>\n\
            </testcase>\n\
        </testsuite>'
        shouldbe_xml = ET.fromstring(str_should_be)
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "PASS", "duration": 20}))
        results.append(Result({"testcase": "test-case-A2", "verdict": "PASS", "duration": 50}))
        results.append(Result({"testcase": "test-case-A3", "verdict": "PASS", "duration": 120}))
        results.append(Result({"testcase": "test-case-A4",
                               "verdict": "FAIL", "reason": "unknown", "duration": 120}))
        results.append(Result({"testcase": "test-case-A5",
                               "verdict": "SKIP", "reason": "unknown", "duration": 1}))
        results.append(Result({"testcase": "test-case-A6",
                               "verdict": "INCONCLUSIVE", "reason": "unknown", "duration": 2}))
        results.append(
            Result({"testcase": "test-case-A4",
                    "verdict": "FAIL", "reason": "WIN blue screen", "duration": 1220}))
        junit = ReportJunit(results)
        str_report = junit.to_string()
        is_xml = ET.fromstring(str_report)
        self.assertDictEqual(is_xml.attrib, {"failures": "2", "tests": "7", "errors": "1",
                                             "skipped": "1"})
        self.assertEqual(len(is_xml.findall("testcase")), len(shouldbe_xml.findall("testcase")))
        self.assertEqual(len(is_xml.findall("failure")), len(shouldbe_xml.findall("failure")))
        self.assertEqual(len(is_xml.findall("skipped")), len(shouldbe_xml.findall("skipped")))
        self.assertEqual(len(is_xml.findall("error")), len(shouldbe_xml.findall("error")))

    def test_junit_hex_escape_support(self):
        reprstring = hex_escape_str(b'\x00\x00\x00\x00\x00\x00\x01\xc8')
        str_should_be = '<testsuite failures="1" tests="1" errors="0" skipped="0">\n\
            <testcase classname="test-case-A1" name="unknown" time="20">\n\
                <failure message="' + reprstring + '"></failure>\n\
            </testcase>\n\
        </testsuite>'
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "FAIL",
                               "reason": b'\x00\x00\x00\x00\x00\x00\x01\xc8', "duration": 20}))
        junit = ReportJunit(results)
        str_report = junit.to_string()
        xml_report = ET.fromstring(str_report)
        shouldbe_report = ET.fromstring(str_should_be)
        self.assertDictEqual(xml_report.attrib, shouldbe_report.attrib)
        reported_tc = xml_report.find("testcase")
        failure_reason = reported_tc.find("failure")
        required_tc = shouldbe_report.find("testcase")
        required_reason = required_tc.find("failure")
        self.assertTrue(required_reason.attrib["message"] == failure_reason.attrib["message"])

    def test_junit_hides(self):
        str_should_be1 = '<testsuite failures="0" tests="3" errors="0" skipped="0">\n\
            <testcase classname="test-case-A1" name="unknown" time="20"></testcase>\n\
            <testcase classname="test-case-A4" name="unknown" time="120"></testcase>\n\
            <testcase classname="test-case-A6" name="unknown" time="2"></testcase>\n\
        </testsuite>'
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "PASS", "duration": 20}))
        failres = Result({"testcase": "test-case-A4", "verdict": "FAIL", "reason": "unknown",
                          "duration": 120})
        failres.retries_left = 1
        results.append(failres)
        results.append(
            Result({"testcase": "test-case-A4", "verdict": "PASS", "duration": 120}))
        incres = Result({"testcase": "test-case-A6",
                         "verdict": "INCONCLUSIVE",
                         "reason": "unknown", "duration": 2})
        incres.retries_left = 1
        results.append(incres)
        results.append(Result({"testcase": "test-case-A6", "verdict": "PASS", "duration": 2}))

        junit = ReportJunit(results)
        str_report = junit.to_string()
        report_xml = ET.fromstring(str_report)
        shouldbe_xml = ET.fromstring(str_should_be1)
        self.assertDictEqual(report_xml.attrib, shouldbe_xml.attrib)
        self.assertEqual(len(report_xml.findall("testcase")), len(shouldbe_xml.findall("testcase")))

        str_should_be2 = '<testsuite failures="0" tests="3" errors="1" skipped="0">\n\
            <testcase classname="test-case-A1" name="unknown" time="20"></testcase>\n\
            <testcase classname="test-case-A4" name="unknown" time="12"></testcase>\n\
            <testcase classname="test-case-A6" name="unknown" time="2">\n\
                <error message="unknown"></error>\n\
            </testcase>\n\
        </testsuite>'
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "PASS", "duration": 20}))
        failres = Result({"testcase": "test-case-A4",
                          "verdict": "FAIL", "reason": "unknown", "duration": 120})
        failres.retries_left = 1
        results.append(failres)
        results.append(Result({"testcase": "test-case-A4", "verdict": "PASS", "duration": 12}))
        results.append(
            Result({"testcase": "test-case-A6",
                    "verdict": "INCONCLUSIVE", "reason": "unknown", "duration": 2}))
        junit = ReportJunit(results)
        str_report = junit.to_string()
        report_xml = ET.fromstring(str_report)
        shouldbe_xml = ET.fromstring(str_should_be2)
        self.assertDictEqual(report_xml.attrib, shouldbe_xml.attrib)
        self.assertEqual(len(report_xml.findall("testcase")), len(shouldbe_xml.findall("testcase")))
        errors = []
        for elem in report_xml.findall("testcase"):
            if elem.find("error") is not None:
                errors.append(elem)
        self.assertEqual(len(errors), 1)

    """
    ReportHtml tests
    """

    def test_html_report(self):
        results = ResultList()
        results.append(Result({"testcase": "test-case-A1", "verdict": "PASS", "duration": 20}))
        failres = Result({"testcase": "test-case-A4", "verdict": "FAIL",
                          "reason": "unknown", "duration": 120})
        results.append(failres)
        html_report = ReportHtml(results)
        # pylint: disable=protected-access
        hotml = html_report._create(title='Test Results', heads={'Build': '', 'Branch': ""},
                                    refresh=None)
        doc = html.document_fromstring(hotml)
        body = doc.get_element_by_id("body")
        passes = body.find_class("item_pass")
        fails = body.find_class("item_fail")
        self.assertEqual(len(passes), len(fails))



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

ReportBase module, contains the base class for other report types.
"""

import os
from datetime import timedelta
import icetea_lib.LogManager as LogManager

#pylint: disable=no-self-use

class ReportBase(object):
    """
    ReportBase is the baseclass of all other report types. It implements helpers related to
    reporting as well as the abstract generate-method.
    """
    def __init__(self, results):
        self.results = results
        self.summary = results.get_summary()

    def get_latest_filename(self, extension, basename="../latest."):
        """
        Generate filename with 'latest.' prefix.

        :param extension: Extension for file
        :param basename: Base file name
        :return: path to latest.basename.extension.
        """
        return os.path.join(LogManager.get_base_dir(), basename+extension)

    def get_current_filename(self, extension, basename="result."):
        """
        Generate filename for a report.

        :param extension: Extension for file name
        :param basename: Base file name
        :return: path to basename.extension
        """
        return os.path.join(LogManager.get_base_dir(), basename+extension)

    def generate(self, *args, **kwargs):
        """
        Abstract generate-method.
        """
        raise NotImplementedError("generate function missing")

    def duration_to_string(self, seconds):
        """
        Converts time in seconds to a timedelta and represents it as string.

        :param seconds: Time in seconds
        :return: str(datetime.timedelta)
        """
        delta = timedelta(seconds=seconds)
        return str(delta)



# pylint: disable=unused-argument

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


LocalAllocator module. Implements allocating local resources using mbedls
"""

import logging

from icetea_lib.LogManager import get_resourceprovider_logger, set_level
from icetea_lib.AllocationContext import AllocationContext, AllocationContextList
from icetea_lib.ResourceProvider.Allocators.BaseAllocator import BaseAllocator
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError
from icetea_lib.ResourceProvider.exceptions import ResourceInitError

from icetea_lib.Plugin.plugins.LocalAllocator.DutDetection import DutDetection
from icetea_lib.Plugin.plugins.LocalAllocator.DutConsole import DutConsole
from icetea_lib.Plugin.plugins.LocalAllocator.DutProcess import DutProcess
from icetea_lib.Plugin.plugins.LocalAllocator.DutSerial import DutSerial


def init_hardware_dut(contextlist, conf, index, args):
    """
    Initializes a local hardware dut
    """
    binary = None
    try:
        binary = conf["application"]['bin']
    except KeyError:
        pass

    dev = conf.get('allocated', None)
    if not dev:
        raise ResourceInitError("Allocated device not found.")

    port = dev['serial_port']
    baudrate = (args.baudrate if args.baudrate else
                conf.get("application", {}).get("baudrate", 115200))
    serial_config = {}
    if args.serial_rtscts:
        serial_config["serial_rtscts"] = args.serial_rtscts
    elif args.serial_xonxoff:
        serial_config["serial_xonxoff"] = args.serial_xonxoff

    if args.serial_timeout:
        serial_config["serial_timeout"] = args.serial_timeout

    ch_mode_config = {}
    if args.serial_ch_size > 0:
        ch_mode_config["ch_mode"] = True
        ch_mode_config["ch_mode_chunk_size"] = args.serial_ch_size
    elif args.serial_ch_size is 0:
        ch_mode_config["ch_mode"] = False

    if args.ch_mode_ch_delay:
        ch_mode_config["ch_mode_ch_delay"] = args.ch_mode_ch_delay

    dut = DutSerial(name="D%d" % index, port=port, baudrate=baudrate,
                    config=conf, ch_mode_config=ch_mode_config,
                    serial_config=serial_config, params=args)
    dut.index = index

    # If something to flash, get the allocated device and flash it
    if binary and not args.skip_flash:
        if contextlist.check_flashing_need('hardware',
                                           binary,
                                           args.forceflash):
            if dut.flash(binary=binary, forceflash=args.forceflash):
                contextlist.logger.info('flash ready')
            else:
                dut.close_dut(False)
                dut.close_connection()
                dut = None
                raise ResourceInitError("Dut flashing failed!")

    dut.platform = dev["platform_name"]

    msg = 'Use board {} as "{}" (id: {})'
    contextlist.logger.info(msg.format(dev['platform_name'],
                                       dut.get_dut_name(),
                                       dev['target_id']))

    contextlist.duts.append(dut)
    contextlist.dutinformations.append(dut.get_info())


def init_process_dut(contextlist, conf, index, args):
    """
    Initialize process type Dut as DutProcess or DutConsole.
    """
    if "subtype" in conf and conf["subtype"]:
        if conf["subtype"] != "console":
            msg = "Unrecognized process subtype: {}"
            contextlist.logger.error(msg.format(conf["subtype"]))
            raise ResourceInitError("Unrecognized process subtype: {}")
        # This is a specialized 'console' process
        config = None
        if "application" in conf:
            config = conf["application"]
        contextlist.logger.debug("Starting a remote console")
        dut = DutConsole(name="D%d" % index, conf=config, params=args)
        dut.index = index
    else:
        binary = conf["application"]['bin']
        app_config = conf["application"]
        init_cli_cmds = app_config.get("init_cli_cmds", None)
        post_cli_cmds = app_config.get("post_cli_cmds", None)
        contextlist.logger.debug("Starting process '%s'" % binary)
        dut = DutProcess(name="D%d" % index, config=conf, params=args)
        dut.index = index
        dut.command = binary
        if args.valgrind:
            dut.use_valgrind(args.valgrind_tool,
                             not args.valgrind_text,
                             args.valgrind_console,
                             args.valgrind_track_origins,
                             args.valgrind_extra_params)
        if args.gdb == index:
            dut.use_gdb()
            contextlist.logger.info("GDB is activated for node %i" % index)
        if args.gdbs == index:
            dut.use_gdbs(True, args.gdbs_port)
            contextlist.logger.info("GDBserver is activated for node %i" % index)
        if args.vgdb == index:
            dut.use_vgdb()
            contextlist.logger.info("VGDB is activated for node %i" % index)
        if args.nobuf:
            dut.no_std_buf()

        if init_cli_cmds is not None:
            dut.set_init_cli_cmds(init_cli_cmds)
        if post_cli_cmds is not None:
            dut.set_post_cli_cmds(post_cli_cmds)

    contextlist.duts.append(dut)
    contextlist.dutinformations.append(dut.get_info())


class LocalAllocator(BaseAllocator):
    """
    LocalAllocator class, subclasses BaseAllocator. Implements allocation of local resources for
    use in test cases. Uses mbedls to detect mbed devices.
    """
    def __init__(self, args=None, logger=None, allocator_cfg=None):
        super(LocalAllocator, self).__init__()
        self.logger = logger
        if self.logger is None:
            self.logger = get_resourceprovider_logger("LocalAllocator", "LAL")
            set_level("LAL", logging.DEBUG)
        self._available_devices = []

    @property
    def share_allocations(self):
        """
        Just return False, allocation sharing not implemented for this allocator.

        :return: False
        """
        return False

    def can_allocate(self, dut_configuration):
        """
        Checks if resource type is supported.
        :param dut_configuration: ResourceRequirements object
        :return: True if type is supported, False otherwise
        """
        try:
            return dut_configuration["type"] in ["hardware", "process"]
        except KeyError:
            return False

    def allocate(self, dut_configuration_list, args=None):
        """
        Allocates resources from available local devices.

        :param dut_configuration_list: List of ResourceRequirements objects
        :param args: Not used
        :return: AllocationContextList with allocated resources
        """
        dut_config_list = dut_configuration_list.get_dut_configuration()
        # if we need one or more local hardware duts let's search attached
        # devices using DutDetection
        if not isinstance(dut_config_list, list):
            raise AllocationError("Invalid dut configuration format!")

        if next((item for item in dut_config_list if item.get("type") == "hardware"), False):
            self._available_devices = DutDetection().get_available_devices()
            if len(self._available_devices) < len(dut_config_list):
                raise AllocationError("Required amount of devices not available.")

        # Enumerate all required DUT's
        try:
            for dut_config in dut_config_list:
                if not self.can_allocate(dut_config.get_requirements()):
                    raise AllocationError("Resource type is not supported")
                self._allocate(dut_config.get_requirements())
        except AllocationError:
            # Locally allocated don't need to be released any way for
            # now, so just re-raise the error
            raise
        alloc_list = AllocationContextList()
        res_id = None
        for conf in dut_config_list:
            if conf.get("type") == "hardware":
                res_id = conf.get("allocated").get("target_id")
            context = AllocationContext(resource_id=res_id, alloc_data=conf)
            alloc_list.append(context)

        alloc_list.set_dut_init_function("hardware", init_hardware_dut)
        alloc_list.set_dut_init_function("process", init_process_dut)

        return alloc_list

    def release(self, dut=None):
        """
        Resource releasing is not necessary. Not implemented
        :param dut: Not used
        :return: Nothing
        """
        pass

    def _allocate(self, dut_configuration):
        """
        Internal allocation function. Allocates a single resource based on dut_configuration
        :param dut_configuration: Dictionary which describes a required resource
        :return: True
        :raises: AllocationError if suitable resource was not found or if the platform was not
        allowed to be used.
        """
        if dut_configuration["type"] == "hardware":
            if not self._available_devices:
                raise AllocationError("No available devices to allocate from")
            platforms = None if 'allowed_platforms' not in dut_configuration else dut_configuration[
                'allowed_platforms']
            platform_name = None if 'platform_name' not in dut_configuration else dut_configuration[
                "platform_name"]
            if platform_name is None and platforms:
                platform_name = platforms[0]
            if platform_name and platforms:
                if platform_name not in platforms:
                    raise AllocationError("Platform name not in allowed platforms.")
            # Enumerate through all available devices
            for dev in self._available_devices:
                if platform_name and dev["platform_name"] != platform_name:
                    self.logger.debug("Skipping device %s because of mismatching platform. "
                                      "Required %s but device was %s", dev['target_id'],
                                      platform_name, dev['platform_name'])
                    continue
                if dev['state'] == 'allocated':
                    self.logger.debug("Skipping device %s because it was "
                                      "already allocated", dev['target_id'])
                    continue

                if DutDetection.is_port_usable(dev['serial_port']):
                    dev['state'] = "allocated"
                    dut_configuration['allocated'] = dev
                    self.logger.info("Allocated device %s", dev['target_id'])
                    return True
                else:
                    self.logger.info("Could not open serial port (%s) of "
                                     "allocated device %s", dev['serial_port'], dev['target_id'])
            # Didn't find a matching device to allocate so allocation failed
            raise AllocationError("No suitable local device available")
        # Successful allocation, return True

        return True



# Disable "too many instance attributes" and "too few public methods" warnings
# pylint: disable=R0902,R0903

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

CliRequest module. Contains CliRequest class.

"""

import time


# Command Request
class CliRequest(object):
    """
    CliRequest class. This is a command request object, that contains the command,
    creation timestamp and other values related to the command sent to a dut.
    """

    def __init__(self, cmd="", timestamp=time.time(), **kwargs):
        """
        Constuctor for request.

        :param cmd: Command sent as string
        :param timestamp: timestamp value, default is time.time() called when creating this object.
        :param kwargs: Keyword arguments. Used arguments are: wait, expected_retcode, timeout,
        These values will populate class members that share the same name.
        """

        self.cmd = cmd
        self.wait = True
        self.timeout = 10
        self.asynchronous = False
        self.timestamp = timestamp
        self.expected_retcode = 0
        self.response = None
        self.dut_index = -1

        for key in kwargs:
            if key == 'wait':
                self.wait = kwargs[key]
            elif key == 'expected_retcode':
                self.expected_retcode = kwargs[key]
            elif key == 'timeout':
                self.timeout = kwargs[key]
            elif key == 'asynchronous':
                self.asynchronous = kwargs[key]
            elif key == 'dut_index':
                self.dut_index = kwargs[key]

    def __str__(self):
        return self.cmd

    def get_timedelta(self, now):
        """
        Return time difference to now from the start of this Request.
        :param now: Timestamp to which time difference should be calculated to.
        :return: Result of calculation as integer.
        """
        return now-self.timestamp



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from icetea_lib.tools.tools import hex_escape_str, import_module, check_int
from icetea_lib.tools.tools import combine_urls, test_case, flush_queue, get_abs_path
from icetea_lib.tools.tools import generate_object_graphs_by_class, num, is_pid_running
from icetea_lib.tools.tools import get_fw_name, get_fw_version, get_number
from icetea_lib.tools.tools import recursive_dictionary_get, get_pkg_version
from icetea_lib.tools.tools import sha1_of_file, strip_escape, load_class
from icetea_lib.tools.tools import remove_empty_from_dict, Singleton
from icetea_lib.tools.tools import UNIXPLATFORM, IS_PYTHON3, set_or_delete, split_by_n


"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from icetea_lib.Plugin.plugins.HttpApi import Api as tc_api
from icetea_lib.TestStepError import TestStepFail


# Schema to make sure header fields are overwritten
schema = {
    "properties": {
        "mergeStrategy": "overwrite"
    }
}


class MockedRequestsResponse():
    def __init__(self, status_code=200, json_data={"key1": "value1"}):
        self.json_data = json_data
        self.status_code = status_code
        self.url = ''
        self.headers = {"head": "ers"}
        self.text = "This is test text"
        self.request = self

    def json(self):
        return self.json_data


class APITestCase(unittest.TestCase):

    def setUp(self):
        self.host = "http://somesite.com"

    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.get", side_effect=iter([
        MockedRequestsResponse(
        status_code=200), MockedRequestsResponse(status_code=404),
        MockedRequestsResponse(status_code=404)]))
    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.put", side_effect=iter([
        MockedRequestsResponse(
        status_code=200), MockedRequestsResponse(status_code=404),
        MockedRequestsResponse(status_code=404)]))
    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.post", side_effect=iter([
        MockedRequestsResponse(
        status_code=200), MockedRequestsResponse(status_code=404),
        MockedRequestsResponse(status_code=404)]))
    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.delete", side_effect=iter([
        MockedRequestsResponse(
        status_code=200), MockedRequestsResponse(status_code=404),
        MockedRequestsResponse(status_code=404)]))
    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.patch", side_effect=iter([
        MockedRequestsResponse(
        status_code=200), MockedRequestsResponse(status_code=404),
        MockedRequestsResponse(status_code=404)]))
    def test_tcapi(self, mock_patch, mock_delete, mock_post, mock_put, mock_get):
        self.http = tc_api(host=self.host, headers=None, cert=None, logger=None)
        self.assertEquals(self.http.get("/").status_code, 200)
        self.assertEquals(self.http.get("/", expected=200, raiseException=False).status_code, 404)
        with self.assertRaises(TestStepFail):
            self.http.get("/", expected=200, raiseException=True)

        path = "/test"
        data = {"testkey1": "testvalue1"}
        self.assertEquals(self.http.put(path, data).status_code, 200)
        self.assertEquals(self.http.put(path, data, expected=200, raiseException=False).status_code, 404)
        with self.assertRaises(TestStepFail):
            self.http.put(path, data,  expected=200, raiseException=True)

        self.assertEquals(self.http.post(path, json=data).status_code, 200)
        self.assertEquals(self.http.post(path, json=data, expected=200, raiseException=False).status_code, 404)
        with self.assertRaises(TestStepFail):
            self.http.post(path, json=data, expected=200, raiseException=True)

        self.assertEquals(self.http.delete(path).status_code, 200)
        self.assertEquals(self.http.delete(path, expected=200, raiseException=False).status_code, 404)
        with self.assertRaises(TestStepFail):
            self.http.delete(path, expected=200, raiseException=True)

        self.assertEquals(self.http.patch(path, data=data).status_code, 200)
        self.assertEquals(self.http.patch(path, data=data, expected=200, raiseException=False).status_code, 404)
        with self.assertRaises(TestStepFail):
            self.http.patch(path, data=data, expected=200, raiseException=True)


if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

import mock

from icetea_lib.CliResponse import CliResponse
from icetea_lib.DeviceConnectors.Dut import Dut
from icetea_lib.TestStepError import TestStepError, TestStepTimeout


@mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager")
class DutTestcase(unittest.TestCase):

    def test_executionready_wait_skip(self, mock_log):  # pylint: disable=unused-argument
        dut = Dut("test_dut")
        with mock.patch.object(dut, "response_received") as mock_r:
            mock_r.wait = mock.MagicMock()
            mock_r.wait.return_value = False
            dut.query_timeout = 0
            dut.response_coming_in = -1

            with self.assertRaises(TestStepError):
                dut._wait_for_exec_ready()  # pylint: disable=protected-access

            mock_r.wait.return_value = True
            dut.query_timeout = 1

            with self.assertRaises(TestStepError):
                dut._wait_for_exec_ready()  # pylint: disable=protected-access

    def test_execready_wait_timeout(self, mock_log):  # pylint: disable=unused-argument
        dut = Dut("test_dut")
        with mock.patch.object(dut, "response_received") as mock_r:
            with mock.patch.object(dut, "get_time", return_value=2):
                mock_r.wait = mock.MagicMock()
                mock_r.wait.return_value = False
                dut.response_coming_in = -1
                dut.query_timeout = 1
                with self.assertRaises(TestStepTimeout):
                    dut._wait_for_exec_ready()  # pylint: disable=protected-access

    def test_initclihuman(self, mock_log):  # pylint: disable=unused-argument
        dut = Dut("test_Dut")
        with mock.patch.object(dut, "execute_command") as m_com:
            dut.post_cli_cmds = None
            dut.init_cli_human()
            self.assertListEqual(dut.post_cli_cmds, dut.set_default_init_cli_human_cmds())
            self.assertEqual(len(m_com.mock_calls), len(dut.post_cli_cmds))

            m_com.reset_mock()
            dut.post_cli_cmds = ["com1", "com2"]
            dut.init_cli_human()
            self.assertListEqual(dut.post_cli_cmds, ["com1", "com2"])
            self.assertEqual(len(m_com.mock_calls), 2)

            m_com.reset_mock()
            dut.post_cli_cmds = [["com1", True, False]]
            dut.init_cli_human()
            self.assertListEqual(dut.post_cli_cmds, [["com1", True, False]])
            self.assertEqual(len(m_com.mock_calls), 1)
            m_com.assert_called_once_with("com1", wait=False, asynchronous=True)

    def test_dut_run(self, mock_log):
        dut = Dut("test_Dut")
        type(dut)._logger = mock.MagicMock()
        dut.logger = mock.MagicMock()
        semaphore = mock.MagicMock()
        semaphore.acquire = mock.MagicMock(side_effect=[True, Exception])

        type(dut)._sem = semaphore
        type(dut)._signalled_duts = [dut]
        type(dut)._run = mock.MagicMock(side_effect=[True, False])
        type(dut).waiting_for_response = mock.PropertyMock(return_value=None)

        query = mock.MagicMock()
        query.cmd = mock.MagicMock()
        dut.query = query
        dut.writeline = mock.MagicMock(side_effect=[Exception])
        dut.readline = mock.MagicMock(return_value=False)
        dut.response_received = mock.MagicMock()
        with self.assertRaises(Exception):
            dut.run()

    def test_store_traces(self, mock_log):
        dut = Dut("test")
        self.assertTrue(dut.store_traces)
        dut.store_traces = False
        self.assertFalse(dut.store_traces)
        dut.store_traces = True
        self.assertTrue(dut.store_traces)

    def test_read_reasponse(self, mock_log):
        dut = Dut("test")
        with mock.patch.object(dut, "readline") as mocked_readline:
            mocked_readline.side_effect = ["retcode: 0", RuntimeError, "retcode: 0", "test"]
            resp = dut._read_response()
            self.assertTrue(isinstance(resp, CliResponse))
            self.assertEqual(dut._read_response(), -1)
            prior_len = len(dut.traces)
            dut.store_traces = False
            dut._read_response()
            self.assertEqual(len(dut.traces), prior_len)
            dut.store_traces = True
            resp = dut._read_response()
            self.assertEqual(len(dut.traces), prior_len + 1)
            self.assertIsNone(resp)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


Module for the HttpApi default plugin as well as a wrapper-class for the api.
"""

from icetea_lib.Plugin.PluginBase import PluginBase
from icetea_lib.TestStepError import TestStepFail
import icetea_lib.tools.HTTP.Api as HttpApi


class HttpApiPlugin(PluginBase):
    """
    Plugin class implementation
    """
    def __init__(self):
        super(HttpApiPlugin, self).__init__()
        self.logger = None
        self.bench = None

    def init(self, bench=None):
        self.bench = bench
        if self.bench is None:
            raise AttributeError("Bench instance not present!")
        if hasattr(bench, "logger"):
            self.logger = bench.logger

    def get_bench_api(self):
        return {"HttpApi": self.get_tc_api}

    def get_tc_api(self, host, headers=None, cert=None, logger=None):
        '''
        Gets HttpApi wrapped into a neat little package that raises TestStepFail
        if expected status code is not returned by the server.
        Default setting for expected status code is 200. Set expected to None when calling methods
        to ignore the expected status code parameter or
        set raiseException = False to disable raising the exception.
        '''
        if logger is None and self.logger:
            logger = self.logger
        return Api(host, headers, cert, logger)


class Api(HttpApi.HttpApi):
    """
    Wrapper for HttpApi.
    """
    def __init__(self, host, headers, cert, logger):  # pylint: disable=useless-super-delegation
        super(Api, self).__init__(host, headers, cert, logger)

    def _raise_fail(self, response, expected):
        """
        Raise a TestStepFail with neatly formatted error message
        """
        try:
            if self.logger:
                self.logger.error("Status code "
                                  "{} != {}. \n\n "
                                  "Payload: {}".format(response.status_code,
                                                       expected,
                                                       response.content))
            raise TestStepFail("Status code {} != {}.".format(response.status_code, expected))
        except TestStepFail:
            raise
        except:  # pylint: disable=bare-except
            if self.logger:
                self.logger.error("Status code "
                                  "{} != {}. \n\n "
                                  "Payload: {}".format(response.status_code,
                                                       expected,
                                                       "Unable to parse payload"))
            raise TestStepFail("Status code {} != {}.".format(response.status_code, expected))

    # pylint: disable=arguments-differ,too-many-arguments
    def get(self, path, headers=None, params=None, expected=200, raiseException=True, **kwargs):
        response = super(Api, self).get(path, headers, params, **kwargs)
        if expected is not None and response.status_code != expected:
            if raiseException:
                self._raise_fail(response, expected)
        return response

    def post(self, path, data=None, json=None, headers=None, expected=200,
             raiseException=True, **kwargs):
        response = super(Api, self).post(path, data, json, headers, **kwargs)
        if expected is not None and response.status_code != expected:
            if raiseException:
                self._raise_fail(response, expected)
        return response

    def put(self, path, data=None, headers=None, expected=200, raiseException=True, **kwargs):
        response = super(Api, self).put(path, data, headers, **kwargs)
        if expected is not None and response.status_code != expected:
            if raiseException:
                self._raise_fail(response, expected)
        return response

    def delete(self, path, headers=None, expected=200, raiseException=True, **kwargs):
        response = super(Api, self).delete(path, headers, **kwargs)
        if expected is not None and response.status_code != expected:
            if raiseException:
                self._raise_fail(response, expected)
        return response

    def patch(self, path, data=None, headers=None, expected=200, raiseException=True, **kwargs):
        response = super(Api, self).patch(path, data, headers, **kwargs)
        if expected is not None and response.status_code != expected:
            if raiseException:
                self._raise_fail(response, expected)
        return response



# pylint: disable=missing-docstring,protected-access

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import math
import mock
from icetea_lib.ResourceProvider.ResourceConfig import ResourceConfig
from icetea_lib.ResourceProvider.ResourceRequirements import ResourceRequirements

class TestVerify(unittest.TestCase):
    def setUp(self):
        self.resconfig = ResourceConfig()
        self.testreqs = {
            "requirements": {
                "duts": {
                    "*": {"count": 2, "type": "process"}
                }
            }
        }
        self.testreqsresult = [
            {
                "type": "process",
                "allowed_platforms": [],
                "nick": None,
            },
            {
                "type": "process",
                "allowed_platforms": [],
                "nick": None,
            }
        ]

    def tearDown(self):
        del self.resconfig

    def test_dut_counts(self):
        self.resconfig._dut_requirements = [ResourceRequirements({"type": "process"}),
                                            ResourceRequirements({"type": "hardware"})]
        self.resconfig._resolve_dut_count()
        self.assertEqual(self.resconfig.count_duts(), 2)
        self.assertEqual(self.resconfig.count_hardware(), 1)
        self.assertEqual(self.resconfig.count_process(), 1)

        self.resconfig._dut_requirements = [ResourceRequirements({"type": "process"}),
                                            ResourceRequirements({"type": "process"}),
                                            ResourceRequirements({"type": "hardware"})]
        self.resconfig._resolve_dut_count()
        self.assertEqual(self.resconfig.count_duts(), 3)
        self.assertEqual(self.resconfig.count_hardware(), 1)
        self.assertEqual(self.resconfig.count_process(), 2)

        self.resconfig._dut_requirements = []
        self.resconfig._resolve_dut_count()
        self.assertEqual(self.resconfig.count_duts(), 0)
        self.assertEqual(self.resconfig.count_hardware(), 0)
        self.assertEqual(self.resconfig.count_process(), 0)

    def test_dut_counts_invalid_fields(self):
        self.resconfig._dut_requirements = [ResourceRequirements({}),
                                            ResourceRequirements({"type": "process"}),
                                            ResourceRequirements({"type": "hardware"})]
        self.assertRaises(ValueError, self.resconfig._resolve_dut_count)

    def test_resolve_requirements(self):
        self.resconfig.resolve_configuration(self.testreqs)
        self.assertEqual(self.resconfig.get_dut_configuration()[0].get_requirements(),
                         self.testreqsresult[0])
        self.assertEqual(self.resconfig.get_dut_configuration()[1].get_requirements(),
                         self.testreqsresult[1])

    def test_dut_requirements_single(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 1,
                            "nick": None
                        },
                        1: {
                            "nick": "Router{i}"
                        }
                    }
                }
            }
        )
        self.assertEqual([self.resconfig.get_dut_configuration()[0].get_requirements()], [
            {"type": "hardware", "allowed_platforms": [], "nick": "Router1"}
        ])

    def test_dut_reqs_three_duts(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 3,
                            "allowed_platforms": ["K64F"],
                            "application": {"bin": "hex.bin"}
                        },
                        1: {"application": {"bin": "my_hex.bin"}},
                        2: {"application": {"bin": "my_hex2.bin"}}
                    }
                }
            }
        )
        self.assertEqual([req.get_requirements() for req in self.resconfig.get_dut_configuration()],
                         [
                             {"type": "hardware",
                              "allowed_platforms": ["K64F"],
                              "application": {"bin": "my_hex.bin"},
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": ["K64F"],
                              "application": {"bin": "my_hex2.bin"},
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": ["K64F"],
                              "application": {"bin": "hex.bin"},
                              "nick": None
                             }
                         ]
                        )

    def test_dut_reqs_multiple_duts(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 100,
                            "allowed_platforms": ["K64F"],
                            "application": {"bin": "hex.bin"}
                            },
                        "1..50": {"application": {"bin": "my_hex.bin"}, "nick": "DUT{i}"},
                        51: {"application": {"bin": "my_hex2.bin"}, "nick": "leader"},
                        "52..100": {"application": {"bin": "my_hex3.bin"}}
                    }
                }
            }
        )
        self.assertEqual(len(self.resconfig.get_dut_configuration()), 100)
        self.assertEqual(self.resconfig.get_dut_configuration()[0].get_requirements(),
                         {"type": "hardware", "allowed_platforms": ["K64F"],
                          "application": {"bin": "my_hex.bin"}, "nick": "DUT1"})
        self.assertEqual(self.resconfig.get_dut_configuration()[50].get_requirements(),
                         {"type": "hardware", "allowed_platforms": ["K64F"],
                          "application": {"bin": "my_hex2.bin"}, "nick": "leader"})
        self.assertEqual(self.resconfig.get_dut_configuration()[51].get_requirements(),
                         {"type": "hardware", "allowed_platforms": ["K64F"],
                          "application": {"bin": "my_hex3.bin"}, "nick": None})

    def test_dut_reqs_locations(self):
        self.resconfig.resolve_configuration(
            {"requirements": {
                "duts": {
                    "*": {"count": 1},
                    1: {"location": [1, "{i}+{n}*{xy}"]}
                }
            }
            }
        )
        self.assertEqual([req.get_requirements() for req in self.resconfig.get_dut_configuration()],
                         [{"type": "hardware", "allowed_platforms": [],
                           "location": [1, 2], "nick": None}])

    def test_dut_reqs_locations_i(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {"count": 5},
                        "1..5": {"location": [1, "{i}"]}
                    }
                }
            }
        )
        self.assertEqual([req.get_requirements() for req in self.resconfig.get_dut_configuration()],
                         [
                             {"type": "hardware",
                              "allowed_platforms": [],
                              "location": [1, 1],
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": [],
                              "location": [1, 2],
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": [],
                              "location": [1, 3],
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": [],
                              "location": [1, 4],
                              "nick": None
                             },
                             {"type": "hardware",
                              "allowed_platforms": [],
                              "location": [1, 5],
                              "nick": None
                             }
                         ]
                        )

    def test_reqs_locations_circle(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {"count": 1},
                        1: {"location": ["10*math.cos((3.5*{i})*({pi}/180))",
                                         "10*math.sin((3.5*{i})*({pi}/180))"]}
                    }
                }
            })
        self.assertEqual([req.get_requirements() for req in self.resconfig.get_dut_configuration()],
                         [{"type": "hardware", "allowed_platforms": [], "location": [
                             eval("10*math.cos((3.5*1)*({pi}/180))".replace("{pi}", str(math.pi))),
                             eval("10*math.sin((3.5*1)*({pi}/180))".replace("{pi}", str(math.pi)))],
                             "nick": None}])

    def test_dut_reqs_invalid_location(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {"count": 1},
                        1: {"location": ["1+/", "+/"]}
                    }
                }
            }
        )

    def test_dut_reqs_invalid_loc_len(self):
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {"count": 1},
                        1: {"location": ["1+1"]}
                    }
                }
            }
        )

    def test_dut_reqs_any_indexed_regression(self):  # pylint: disable=invalid-name
        self.resconfig.resolve_configuration(
            {
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 2,
                            "allowed_platforms": [],
                            "type": "hardware"
                        },
                        "1": {
                            "allowed_platforms": ["Tplat1"],
                            "custom_key": "value"
                        },
                        "2": {
                            "allowed_platforms": ["Tplat2"],
                            "custom_key2": "value"
                        }
                    }
                }
            }
        )
        reqs = self.resconfig.get_dut_configuration()
        self.assertDictEqual(reqs[0].get_requirements(), {"allowed_platforms": ["Tplat1"],
                                                          "custom_key": "value",
                                                          "type": "hardware", "nick": None})
        self.assertDictEqual(reqs[1].get_requirements(), {"allowed_platforms": ["Tplat2"],
                                                          "custom_key2": "value",
                                                          "type": "hardware",
                                                          "nick": None})

    def test_setting_dut_parameters(self):
        self.resconfig.resolve_configuration(self.testreqs)
        conf = self.resconfig.get_dut_configuration(1)
        conf.set("test", "test")
        self.resconfig.set_dut_configuration(1, conf)
        res = self.testreqsresult
        res[1]["test"] = "test"
        self.assertEqual([req.get_requirements() for req in self.resconfig.get_dut_configuration()],
                         res)

    def test_init_with_logger(self):
        logger = mock.Mock()
        testrc = ResourceConfig(logger=logger)
        self.assertEqual(testrc.logger, logger)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

PluginManager class implements searching, importing, handling and registering plugins.
"""

import importlib
import sys
import os

from icetea_lib.Plugin.PluginBase import PluginTypes, PluginBase, RunPluginBase
from icetea_lib.Plugin.plugins.default_plugins import default_plugins


class PluginException(Exception):
    """
    Plugin Exception
    """
    pass


class PluginManager(object): # pylint: disable=too-many-instance-attributes
    """
    class PluginManager. The job of this class is to load and register plugins
    for Icetea.

    Loads modules like we load Testcases. That instance is then used to
    get the plugin contents as a dict.

    The contents of this dict are registered to relevant parts of Icetea
    according to the type of plugin we are dealing with.
    """

    def __init__(self, responseparser=None, bench=None, logger=None):
        self.logger = logger
        if self.logger is None:
            import logging
            self.logger = logging.getLogger("PluginManager")
            if not self.logger.handlers:
                self.logger.addHandler(logging.StreamHandler())
                self.logger.setLevel(logging.INFO)
        self.responseparser = responseparser
        self.bench = bench
        self._external_services = {}
        self._started_services = []
        self.registered_plugins = []
        self._allocators = {}
        self.plugin_types = {PluginTypes.BENCH: self._register_bench_extension,
                             PluginTypes.PARSER: self._register_dataparser,
                             PluginTypes.EXTSERVICE: self._register_external_service,
                             PluginTypes.ALLOCATOR: self._register_allocator}

    def register_tc_plugins(self, plugin_name, plugin_class):
        """
        Loads a plugin as a dictionary and attaches needed parts to correct areas for testing
        parts.

        :param plugin_name: Name of the plugins
        :param plugin_class: PluginBase
        :return: Nothing
        """
        if plugin_name in self.registered_plugins:
            raise PluginException("Plugin %s already registered! Duplicate plugins?", plugin_name)
        self.logger.debug("Registering plugin %s", plugin_name)
        plugin_class.init(bench=self.bench)
        if plugin_class.get_bench_api() is not None:
            register_func = self.plugin_types[PluginTypes.BENCH]
            register_func(plugin_name, plugin_class)
        if plugin_class.get_parsers() is not None:
            register_func = self.plugin_types[PluginTypes.PARSER]
            register_func(plugin_name, plugin_class)
        if plugin_class.get_external_services() is not None:
            register_func = self.plugin_types[PluginTypes.EXTSERVICE]
            register_func(plugin_name, plugin_class)

        self.registered_plugins.append(plugin_name)

    def register_run_plugins(self, plugin_name, plugin_class):
        """
        Loads a plugin as a dictionary and attaches needed parts to correct Icetea run
        global parts.

        :param plugin_name: Name of the plugins
        :param plugin_class: PluginBase
        :return: Nothing
        """
        if plugin_name in self.registered_plugins:
            raise PluginException("Plugin %s already registered! Duplicate plugins?", plugin_name)
        self.logger.debug("Registering plugin %s", plugin_name)
        if plugin_class.get_allocators():
            register_func = self.plugin_types[PluginTypes.ALLOCATOR]
            register_func(plugin_name, plugin_class)
        self.registered_plugins.append(plugin_name)

    def get_allocator(self, allocator_name):
        """
        Get a registered allocator based on allocator_name.

        :param allocator_name: Name of allocator to get
        :return: BaseAllocator
        """
        if allocator_name in self._allocators:
            return self._allocators[allocator_name]
        return None

    def load_default_tc_plugins(self):
        """
        Load default test case level plugins from icetea_lib.Plugin.plugins.default_plugins.

        :return: Nothing
        """
        for plugin_name, plugin_class in default_plugins.items():
            if issubclass(plugin_class, PluginBase):
                try:
                    self.register_tc_plugins(plugin_name, plugin_class())
                except PluginException as error:
                    self.logger.debug(error)
                    continue

    def load_custom_tc_plugins(self, plugin_path=None):
        """
        Load custom test case level plugins from plugin_path.

        :param plugin_path: Path to file, which contains the imports and mapping for plugins.
        :return: None if plugin_path is None or False or something equivalent to those.
        """
        if not plugin_path:
            return
        directory = os.path.dirname(plugin_path)
        sys.path.append(directory)
        modulename = os.path.split(plugin_path)[1]
        # Strip of file extension.
        if "." in modulename:
            modulename = modulename[:modulename.rindex(".")]
        try:
            module = importlib.import_module(modulename)
        except ImportError:
            raise PluginException("Unable to import custom plugin information from {}.".format(
                plugin_path))
        for plugin_name, plugin_class in module.plugins_to_load.items():
            if issubclass(plugin_class, PluginBase):
                try:
                    self.register_tc_plugins(plugin_name, plugin_class())
                except PluginException as error:
                    self.logger.debug(error)
                    continue

    def load_default_run_plugins(self):
        """
        Load default run level plugins from icetea_lib.Plugin.plugins.default_plugins.

        :return: Nothing
        """
        for plugin_name, plugin_class in default_plugins.items():
            if issubclass(plugin_class, RunPluginBase):
                try:
                    self.register_run_plugins(plugin_name, plugin_class())
                except PluginException as error:
                    self.logger.debug(error)
                    continue

    def load_custom_run_plugins(self, plugin_path=None):
        """
        Load custom run level plugins from plugin_path.

        :param plugin_path: Path to file, which contains the imports and mapping for plugins.
        :return: None if plugin_path is None or False or something equivalent to those.
        """
        if not plugin_path:
            return
        directory = os.path.dirname(plugin_path)
        sys.path.append(directory)
        modulename = os.path.split(plugin_path)[1]
        # Strip of file extension.
        if "." in modulename:
            modulename = modulename[:modulename.rindex(".")]
        try:
            module = importlib.import_module(modulename)
        except ImportError:
            raise PluginException("Unable to import custom plugin information from {}.".format(
                plugin_path))
        for plugin_name, plugin_class in module.plugins_to_load.items():
            if issubclass(plugin_class, RunPluginBase):
                try:
                    self.register_run_plugins(plugin_name, plugin_class())
                except PluginException as error:
                    self.logger.debug(error)
                    continue

    def start_external_service(self, service_name, conf=None):
        """
        Start external service service_name with configuration conf.

        :param service_name: Name of service to start
        :param conf:
        :return: nothing
        """
        if service_name in self._external_services:
            ser = self._external_services[service_name]
            service = ser(service_name, conf=conf, bench=self.bench)
            try:
                service.start()
            except PluginException:
                self.logger.exception("Starting service %s caused an exception!", service_name)
                raise PluginException("Failed to start external service %s", service_name)
            self._started_services.append(service)
            setattr(self.bench, service_name, service)
        else:
            self.logger.warning("Service %s not found. Check your plugins.", service_name)

    def stop_external_services(self):
        """
        Stop all external services.

        :return: Nothing
        """
        for service in self._started_services:
            self.logger.debug("Stopping application %s", service.name)
            try:
                service.stop()
            except PluginException:
                self.logger.exception("Stopping external service %s caused and exception!",
                                      service.name)
        self._started_services = []

    def _register_bench_extension(self, plugin_name, plugin_instance):
        """
        Register a bench extension.

        :param plugin_name: Plugin name
        :param plugin_instance: PluginBase
        :return: Nothing
        """
        for attr in plugin_instance.get_bench_api().keys():
            if hasattr(self.bench, attr):
                raise PluginException("Attribute {} already exists in bench! Unable to add "
                                      "plugin {}.".format(attr, plugin_name))
            setattr(self.bench, attr, plugin_instance.get_bench_api().get(attr))

    def _register_dataparser(self, plugin_name, plugin_instance):
        """
        Register a parser.

        :param plugin_name: Parser name
        :param plugin_instance: PluginBase
        :return: Nothing
        """
        for parser in plugin_instance.get_parsers().keys():
            if self.responseparser.has_parser(parser):
                raise PluginException("Parser {} already registered to parsers! Unable to "
                                      "add parsers from {}.".format(parser, plugin_name))
            self.responseparser.add_parser(parser, plugin_instance.get_parsers().get(parser))

    def _register_external_service(self, plugin_name, plugin_instance):
        """
        Register an external service.

        :param plugin_name: Service name
        :param plugin_instance: PluginBase
        :return:
        """
        for attr in plugin_instance.get_external_services().keys():
            if attr in self._external_services:
                raise PluginException("External service with name {} already exists! Unable to add "
                                      "services from plugin {}.".format(attr, plugin_name))
            self._external_services[attr] = plugin_instance.get_external_services().get(attr)

    def _register_allocator(self, plugin_name, plugin_instance):
        """
        Register an allocator.

        :param plugin_name: Allocator name
        :param plugin_instance: RunPluginBase
        :return:
        """
        for allocator in plugin_instance.get_allocators().keys():
            if allocator in self._allocators:
                raise PluginException("Allocator with name {} already exists! unable to add "
                                      "allocators from plugin {}".format(allocator, plugin_name))
            self._allocators[allocator] = plugin_instance.get_allocators().get(allocator)



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

import os
import sys

from icetea_lib.Plugin.PluginManager import PluginManager, PluginException
from icetea_lib.Plugin.plugins.default_plugins import default_plugins


class PMTestcase(unittest.TestCase):

    def test_load_defaults(self):
        bench = mock.MagicMock(spec=[])
        bench.logger = mock.MagicMock(return_value=mock.MagicMock())
        rp = mock.MagicMock()
        rp.append = mock.MagicMock()
        rp.has_parser = mock.MagicMock(return_value=False)
        pm = PluginManager(bench=bench, responseparser=rp)
        pm.load_default_tc_plugins()
        pm.load_default_run_plugins()
        length = len(default_plugins)
        self.assertEqual(len(pm.registered_plugins), length)

    def test_register_all_tc_types(self):
        # Set up mocks
        plugin_class = mock.MagicMock()
        plugin_class.init = mock.MagicMock()
        plugin_class.get_bench_api = mock.MagicMock()
        plugin_class.get_parsers = mock.MagicMock()
        plugin_class.get_external_services = mock.MagicMock()
        mock_bench = mock.MagicMock(spec=[])
        mock_bench.logger = mock.MagicMock(return_value=mock.MagicMock())
        mock_bench_function = mock.MagicMock()
        mock_parser = mock.MagicMock()
        plugin_class.get_bench_api.return_value = {"mock_func": mock_bench_function}
        plugin_class.get_external_services.return_value = {"mock_class": mock.MagicMock}
        plugin_class.get_parsers.return_value = {"mock_parser": mock_parser}
        mock_parsermanager = mock.MagicMock()
        mock_parsermanager.add_parser = mock.MagicMock()
        mock_parsermanager.has_parser = mock.MagicMock(return_value=False)

        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.register_tc_plugins("test_plugin", plugin_class)

        # Asserts
        self.assertEqual(len(pm.registered_plugins), 1)
        self.assertEqual(pm.registered_plugins[0], "test_plugin")
        self.assertEqual(len(pm._external_services), 1)
        mock_parsermanager.has_parser.assert_called_once_with("mock_parser")
        mock_parsermanager.add_parser.assert_called_once_with("mock_parser", mock_parser)

    def test_register_and_start_service(self):
        # Set up mocks
        plugin_class = mock.MagicMock()
        plugin_class.init = mock.MagicMock()
        plugin_class.get_bench_api = mock.MagicMock()
        plugin_class.get_parsers = mock.MagicMock()
        plugin_class.get_external_services = mock.MagicMock()

        mock_bench = mock.MagicMock(spec=[])
        mock_bench.logger = mock.MagicMock(return_value=mock.MagicMock())

        plugin_class.get_bench_api.return_value = None
        mock_class = mock.MagicMock()
        plugin_class.get_external_services.return_value = {"mock_class": mock_class}
        plugin_class.get_parsers.return_value = None

        mock_parsermanager = mock.MagicMock()

        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.register_tc_plugins("test_plugin", plugin_class)
        pm.start_external_service("mock_class")
        self.assertEqual(len(pm._started_services), 1)
        pm.stop_external_services()

        self.assertEqual(len(pm._started_services), 0)
        self.assertEqual(len(pm._external_services), 1)
        mock_class.assert_called_once()

    def test_start_service_raises_exception(self):
        # Set up mocks
        plugin_class = mock.MagicMock()
        plugin_class.init = mock.MagicMock()
        plugin_class.get_bench_api = mock.MagicMock()
        plugin_class.get_parsers = mock.MagicMock()
        plugin_class.get_external_services = mock.MagicMock()

        mock_bench = mock.MagicMock(spec=[])
        mock_bench.logger = mock.MagicMock(return_value=mock.MagicMock())

        plugin_class.get_bench_api.return_value = None
        mocked_service = mock.MagicMock()
        mock_class = mock.MagicMock(return_value=mocked_service)
        mocked_service.start = mock.MagicMock()
        mocked_service.start.side_effect = [PluginException]
        plugin_class.get_external_services.return_value = {"mock_class": mock_class}
        plugin_class.get_parsers.return_value = None

        mock_parsermanager = mock.MagicMock()

        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.register_tc_plugins("test_plugin", plugin_class)
        with self.assertRaises(PluginException):
            pm.start_external_service("mock_class")
            mocked_service.start.assert_called_once()

    def test_register_start_stop_service(self):
        plugin_class = mock.MagicMock()
        plugin_class.init = mock.MagicMock()
        plugin_class.get_bench_api = mock.MagicMock()
        plugin_class.get_parsers = mock.MagicMock()
        plugin_class.get_external_services = mock.MagicMock()

        mock_bench = mock.MagicMock(spec=[])
        mock_bench.logger = mock.MagicMock(return_value=mock.MagicMock())

        plugin_class.get_bench_api.return_value = None
        mocked_service = mock.MagicMock()
        mocked_service.start = mock.MagicMock()
        mocked_service.stop = mock.MagicMock(side_effect=[PluginException])
        mock_class = mock.MagicMock(return_value=mocked_service)
        plugin_class.get_external_services.return_value = {"mock_class": mock_class}
        plugin_class.get_parsers.return_value = None

        mock_parsermanager = mock.MagicMock()

        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.register_tc_plugins("test_plugin", plugin_class)
        pm.start_external_service("mock_class")
        self.assertEqual(len(pm._started_services), 1)
        pm.stop_external_services()

        self.assertEqual(len(pm._started_services), 0)
        self.assertEqual(len(pm._external_services), 1)
        mock_class.assert_called_once()

    def test_register_raises_pluginexception(self):
        plugin_class = mock.MagicMock()
        plugin_class.init = mock.MagicMock()
        plugin_class.get_bench_api = mock.MagicMock()

        mock_bench = mock.MagicMock(spec=[])
        mock_bench.logger = mock.MagicMock(return_value=mock.MagicMock())
        mock_bench_function = mock.MagicMock()
        mock_parser = mock.MagicMock()
        plugin_class.get_bench_api.return_value = {"mock_func": mock_bench_function}
        mock_parsermanager = mock.MagicMock()
        mock_parsermanager.add_parser = mock.MagicMock()
        mock_parsermanager.has_parser = mock.MagicMock(return_value=False)

        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.registered_plugins = ["test_plugin"]
        with self.assertRaises(PluginException):
            pm.register_tc_plugins("test_plugin", plugin_class)

    def test_load_custom_plugins(self):
        modules = sys.modules
        mock_bench = mock.MagicMock(spec=[])
        mock_parsermanager = mock.MagicMock()
        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        pm.register_tc_plugins = mock.MagicMock()
        pm.load_custom_tc_plugins(os.path.join(os.path.dirname(os.path.abspath(
            __file__)), "test_plugin/load_test_plugins.py"))
        sys.modules = modules
        pm.register_tc_plugins.assert_called_once()

    @mock.patch("icetea_lib.Plugin.PluginManager.importlib")
    def test_load_custom_plugin_exception(self, mock_importer):
        mock_bench = mock.MagicMock(spec=[])
        mock_parsermanager = mock.MagicMock()
        mock_importer.import_module = mock.MagicMock(side_effect=[ImportError])
        pm = PluginManager(bench=mock_bench, responseparser=mock_parsermanager)
        with self.assertRaises(PluginException):
            pm.load_custom_tc_plugins(os.path.join(os.path.dirname(os.path.abspath(
                __file__)), "test_plugin/load_test_plugins.py"))


if __name__ == '__main__':
    unittest.main()



# pylint: disable=unused-argument,missing-docstring,too-many-arguments,too-few-public-methods
# pylint: disable=no-self-use

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from icetea_lib.Plugin.plugins.LocalAllocator.DutSerial import DutSerial
from icetea_lib.DeviceConnectors.Dut import DutConnectionError

try:
    from mbed_flasher.common import FlashError
except ImportError:
    FlashError = None


class MockArgspec(object):
    def __init__(self, lst):
        self.args = lst


class DutSerialTestcase(unittest.TestCase):

    # Mock base class
    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_flasher_logger_insert(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                                   mock_bench_logger):
        mock_inspect.return_value = MockArgspec(["logger"])
        mocked_logger_for_flasher = mock.MagicMock()
        mocked_logger.return_value = mocked_logger_for_flasher
        dut = DutSerial(port="test",
                        config={"allocated": {"target_id": "thing"}, "application": "thing"})
        dut.flash("this_is_not_a_binary")
        mock_flasher.assert_called_with(logger=mocked_logger_for_flasher)

        mock_flasher.reset_mock()
        mock_inspect.return_value = MockArgspec([])
        dut = DutSerial(port="test",
                        config={"allocated": {"target_id": "thing"}, "application": "thing"})
        dut.flash("this_is_not_a_binary")
        mock_flasher.assert_called_with()

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_flash_build_init_not_implemented(self, mock_build, mock_flasher, mocked_logger,
                                              mock_inspect, mock_bench_logger):
        mock_build.init = mock.MagicMock(side_effect=[NotImplementedError])
        mock_inspect.return_value = MockArgspec(["logger"])
        mocked_logger_for_flasher = mock.MagicMock()
        mocked_logger.return_value = mocked_logger_for_flasher
        dut = DutSerial(port="test",
                        config={"allocated": {"target_id": "thing"}, "application": "thing"})
        with self.assertRaises(DutConnectionError):
            dut.flash("try")

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_flash_build_get_file_fail(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                                       mock_bench_logger):
        mock_build_object = mock.MagicMock()
        mock_build_object.get_file = mock.MagicMock(side_effect=[False, "Thisbin"])
        mock_build.init = mock.MagicMock(return_value=mock_build_object)
        mock_inspect.return_value = MockArgspec(["logger"])
        mocked_logger_for_flasher = mock.MagicMock()
        mocked_logger.return_value = mocked_logger_for_flasher
        dut = DutSerial(port="test",
                        config={"allocated": {"target_id": "thing"}, "application": "thing"})
        with self.assertRaises(DutConnectionError):
            dut.flash("try")

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_flash_fails(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                         mock_bench_logger):
        mock_build_object = mock.MagicMock()
        mock_build_object.get_file = mock.MagicMock(return_value="Thisbin")
        mock_build.init = mock.MagicMock(return_value=mock_build_object)
        mock_inspect.return_value = MockArgspec(["logger"])
        mocked_logger_for_flasher = mock.MagicMock()
        mocked_logger.return_value = mocked_logger_for_flasher
        mock_flasher_object = mock.MagicMock()
        mock_flasher.return_value = mock_flasher_object
        mock_flasher_object.flash = mock.MagicMock()
        mock_flasher_object.flash.side_effect = [NotImplementedError, SyntaxError, 1]
        if FlashError is not None:
            mock_flasher_object.flash.side_effect = [NotImplementedError, SyntaxError, 1,
                                                     FlashError("Error", 10)]
        dut = DutSerial(port="test",
                        config={"allocated": {"target_id": "thing"}, "application": "thing"})
        with self.assertRaises(DutConnectionError):
            dut.flash("try")
        with self.assertRaises(DutConnectionError):
            dut.flash("try2")
        self.assertFalse(dut.flash("try3"))
        if FlashError is not None:
            with self.assertRaises(DutConnectionError):
                dut.flash("try4")

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_flash_skip_flash(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                              mock_bench_logger):
        mock_build_object = mock.MagicMock()
        mock_build_object.get_file = mock.MagicMock(return_value="Thisbin")
        mock_build.init = mock.MagicMock(return_value=mock_build_object)
        mock_inspect.return_value = MockArgspec(["logger"])
        mock_logger_for_flasher = mock.MagicMock()
        mocked_logger.return_value = mock_logger_for_flasher
        mock_flasher_object = mock.MagicMock()
        mock_flasher.return_value = mock_flasher_object
        mock_flasher_object.flash = mock.MagicMock()
        mock_flasher_object.flash.return_value = 0
        dut = DutSerial(port="test", config={
            "allocated": {"target_id": "thing"},
            "application": "thing"
        })
        self.assertTrue(dut.flash("try"))
        self.assertTrue(dut.flash("try"))
        self.assertTrue(dut.flash("try"))
        self.assertEqual(mock_flasher_object.flash.call_count, 1)
        self.assertTrue(dut.flash("try", forceflash=True))
        self.assertEqual(mock_flasher_object.flash.call_count, 2)

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_peek(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                  mock_bench_logger):
        dut = DutSerial(port="test", config={
            "allocated": {"target_id": "thing"},
            "application": "thing"
        })
        dut.port = mock.MagicMock()
        dut.port.peek = mock.MagicMock(return_value="test")
        self.assertEqual(dut.peek(), "test")

    @mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager.get_bench_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.getargspec")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.get_resourceprovider_logger")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Flash")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Build")
    def test_readline(self, mock_build, mock_flasher, mocked_logger, mock_inspect,
                  mock_bench_logger):
        dut = DutSerial(port="test", config={
            "allocated": {"target_id": "thing"},
            "application": "thing"
        })
        dut.port = mock.MagicMock()
        dut.port.readline = mock.MagicMock(side_effect=["test\n", None])
        self.assertEqual(dut._readline(), "test")
        self.assertIsNone(dut._readline())


if __name__ == '__main__':
    unittest.main()


"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

BaseAllocator module, contains the base implementation for an allocator.
"""


class BaseAllocator(object):
    """
    Abstract base class for describing a resource allocator class.
    """
    def can_allocate(self, dut_configuration):
        """
        Returns True if this allocator supports allocating a resource
        matching the given dut_configuration. Returns False if allocating such resource is not
        supported.

        This method has to be implemented by the subclass.

        :param dut_configuration:
        :return: True if this allocator supports allocating
        resources as described in dut_configuration, otherwise False.
        """
        raise NotImplementedError("allocate has to be implemented by subclassing BaseAllocator")

    def allocate(self, dut_configuration_list, args=None):
        """
        Enumerates the dut_configuration_list to evaluate requirements
        for each DUT it contains and attempts to allocate a resource matching the requirements.
        Once all DUT's have a successfully allocated resource, they are
        instantiated by calling dut_factory callable for each dut_configuration contained in
        dut_configuration_list.

        This method has to be implemented by the subclass.

        :param dut_configuration_list: List of dut configuration dictionaries each
        containing a DUT configuration, which has the following generic form:
        {"type": "process", "nick": None }
        :param args: Extra arguments that can be passed to allocate to further
        configure the allocation.
        :raises AllocationError: If not all dut's we're allocated, raises AllocationError
        """
        raise NotImplementedError("allocate has to be implemented by subclassing BaseAllocator")

    def release(self, dut=None):
        """
        Release the allocated resource with the given context.
        This method has to be implemented by the subclass.

        :param allocation_context: The context of the allocated resource.
        :return:
        """
        raise NotImplementedError("release has to be implemented by subclassing BaseAllocator")

    def cleanup(self):
        """
        Cleanup will be called when testcase has been run and allocator should do any necessary
        cleanup operations
        :return:
        """
        pass

    @property
    def share_allocations(self):
        raise NotImplementedError("share_allocations has to be implemented by subclassing "
                                  "BaseAllocator")



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Cloud adapter for Icetea.
"""

import unittest
import os
import copy

from icetea_lib.tools.tools import remove_empty_from_dict, get_pkg_version


def create_result_object(result):
    """
    Create cloud result object from Result.

    :param result: Result
    :return: dictionary
    """
    _result = {
        'tcid': result.get_tc_name(),
        'campaign': result.campaign,
        'cre': {
            'user': result.tester
        },
        'job': {
            'id': result.job_id
        },
        'exec': {
            'verdict': result.get_verdict(),
            'duration': result.duration,
            'note': result.get_fail_reason(),
            'dut': {
                'count': result.dut_count,
                'type': result.dut_type
            },
            'sut': {
                'branch': result.build_branch,
                'commitId': result.buildcommit,
                'buildDate': result.build_date,
                'buildSha1': result.build_sha1,
                'buildUrl': result.build_url,
                'gitUrl': result.build_git_url,
                'cut': result.component,  # Component Under Uest
                'fut': result.feature    # Feature Under Test
            },
            'env': {
                'framework': {
                    'name': result.fw_name,
                    'ver': result.fw_version
                }
            },
            "logs": []
        }
    }

    if result.dut_resource_id:
        _result["exec"]["dut"]["sn"] = result.dut_resource_id

    if result.dut_vendor and result.dut_vendor[0]:
        _result["exec"]["dut"]["vendor"] = result.dut_vendor[0]
    if result.dut_models and result.dut_models[0]:
        _result["exec"]["dut"]["model"] = result.dut_models[0]
    # pylint: disable=len-as-condition
    if len(result.dut_models) == 1 and len(result.dut_resource_id) == 1:
        _result["exec"]["dut"]["sn"] = result.dut_resource_id[0]

    return remove_empty_from_dict(_result)


def append_logs_to_result_object(result_obj, result):
    """
    Append log files to cloud result object from Result.

    :param result_obj: Target result object
    :param result: Result
    :return: Nothing, modifies result_obj in place.
    """
    logs = result.has_logs()
    result_obj["exec"]["logs"] = []
    if logs and result.logfiles:
        for log in logs:
            typ = None
            parts = log.split(os.sep)
            if "bench" in parts[len(parts) - 1]:
                typ = "framework"
            # elif "Dut" in parts[len(parts)-1]:
            #    typ = "dut"

            if typ is not None:
                name = parts[len(parts) - 1]
                try:
                    with open(log, "r") as file_name:
                        data = file_name.read()
                    dic = {"data": data, "name": name, "from": typ}
                    result_obj["exec"]["logs"].append(dic)
                except OSError:
                    pass
            else:
                continue


def create_result_object_with_logs(result):
    """
    Create the result dictionary and append logs as well.
    :param result: Result
    :return: dictionary
    """
    _result = create_result_object(result)
    append_logs_to_result_object(_result, result)
    return _result


# Cloud connection class
class Cloud(object):
    """
    Cloud adapter. Originally designed to work with python client for Opentmi.
    """

    __version = 0
    __api = "/api/v"

    @staticmethod
    def _convert_to_db_tc_metadata(tc_metadata):
        """
        Convert tc_metadata to match Opentmi metadata format
        :param tc_metadata: metadata as dict
        :return: converted metadata
        """
        db_meta = copy.deepcopy(tc_metadata)

        # tcid is a mandatory field, it should throw an error if it is missing
        db_meta['tcid'] = db_meta['name']
        del db_meta['name']

        # Encapsulate current status inside dictionary
        if 'status' in db_meta:
            status = db_meta['status']
            del db_meta['status']
            db_meta['status'] = {'value': status}

        # Node and dut information
        if 'requirements' in db_meta:
            db_meta['requirements']['node'] = {'count': 1}
            try:
                count = db_meta['requirements']['duts']['*']['count']
                db_meta['requirements']['node']['count'] = count
            except KeyError:
                pass

        # Collect and pack other info from meta
        db_meta['other_info'] = {}
        if 'title' in db_meta:
            db_meta['other_info']['title'] = db_meta['title']
            del db_meta['title']

        if 'feature' in db_meta:
            db_meta['other_info']['features'] = db_meta['feature']
            del db_meta['feature']
        else:
            db_meta['other_info']['features'] = ['unknown']

        if 'component' in db_meta:
            db_meta['other_info']['components'] = db_meta["component"]
            del db_meta['component']

        return db_meta

    # pylint: disable=too-many-arguments
    def __init__(self, host=None, module=None, result_converter=None, tc_converter=None,
                 logger=None, args=None):

        self.args = args
        self.logger = logger

        # Try to fetch cloud provider from ENV variables
        if not module:
            module = os.environ.get("ICETEA_CLOUD_PROVIDER", 'opentmi-client')
        self.module = __import__(module, globals(), fromlist=[""])

        version = get_pkg_version(module)
        if self.logger and version is not None:
            self.logger.info("using {} version {}".format(module, version))
        else:
            if self.logger:
                self.logger.warning("Unable to parse cloud module version")

        # Parse host and port from combined host
        if host is None:
            host = self._resolve_host()
        host, port = self._find_port(host)

        # Ensure result converter has an implementation
        resconv = result_converter
        if resconv is None:
            if self.args and self.args.with_logs:
                resconv = create_result_object_with_logs
            else:
                resconv = create_result_object

        # Ensure testcase converter has an implementation
        tc_conv = tc_converter if tc_converter else self._convert_to_db_tc_metadata

        # Setup client
        try:
            self._client = self.module.create(host, port, resconv, tc_conv)
            self._client.set_logger(logger)
        except AttributeError:
            raise ImportError("Cloud module was imported but it does not "
                              "contain a method to create a client.")

    def _resolve_host(self):  # pylint: disable=no-self-use
        """
        Resolve cloud provider host name. Defaults to environment variables
        OPENTMI_ADDRESS_PRIVATE or OPENTMI_ADDRESS_PUBLIC if environment variable NODE_NAME
        starts with 'aws'. Otherwise gets ICETEA_CLOUD_HOST environment variable OR
        localhost:3000 if that one does not exist.
        :return: Cloud host information
        """
        node_name = os.environ.get('NODE_NAME', '')
        if node_name.startswith('aws'):
            _host = os.environ.get('OPENTMI_ADDRESS_PRIVATE', None)
        else:
            _host = os.environ.get('OPENTMI_ADDRESS_PUBLIC', None)

        if _host is None:
            _host = os.environ.get("ICETEA_CLOUD_HOST", "localhost:3000")

        return _host

    def _find_port(self, host):  # pylint: disable=no-self-use
        """
        Finds port number from host. Defaults to 3000 if not found
        :param host: host as string
        :return: (host, port)
        """
        ind = host.rfind(":")
        if ind != -1:
            try:
                port = int(host[ind + 1:])
                host = host[:ind]
            except ValueError:
                port = 3000
        else:
            port = 3000
        return host, port

    def get_suite(self, suite, options=''):
        """
        Calls cloud client method get_suite
        :param suite: passed to cloud client
        :param options: passed to cloud client
        :return: _client.get_suite(suite, options)
        """
        return self._client.get_suite(suite, options)

    def get_campaign_id(self, campaign_name):
        """
        Calls client method get_campaign_id
        :param campaign_name: passed to cloud client
        :return: _client.get_campaign_id(campaign_name)
        """
        return self._client.get_campaign_id(campaign_name)

    def get_campaigns(self):
        """
        Calls client method get_campaigns
        :return: _client.get_campaigns()
        """
        return self._client.get_campaigns()

    def get_campaign_names(self):
        """
        Calls client method get_campaign_names
        :return: _client.get_campaign_names()
        """
        return self._client.get_campaign_names()

    def update_testcase(self, metadata):
        """
        Updates test case metadata with
        :param metadata: Test case metadata
        :return: _client.update_testcase(metadata)
        """
        return self._client.update_testcase(metadata)

    def send_result(self, result):
        """
        Send results to the cloud
        :param result: result dictionary
        :return: response from _client.upload_results(result) or None if something went wrong
        """
        response_data = self._client.upload_results(result)
        if response_data:
            if self.logger is not None:
                self.logger.info("Results sent to the server. ID: {}".format(response_data["_id"]))
            return response_data
        else:
            if self.logger is not None:
                self.logger.info("Server didn't respond or client initialization has failed.")
            return None




if __name__ == '__main__':
    unittest.main()
